@@ -3,37 +3,26 @@
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
 #
-#     If you submit patches or make the software available to licensors of
-#     this software in either form, you automatically them grant them a
-#     license for your part of the code under "Apache License 2.0" unless you
-#     choose to remove this notice.
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
 #
-#     Kay Hayen uses the right to license his code under only GPL version 3,
-#     to discourage a fork of Nuitka before it is "finished". He will later
-#     make a new "Nuitka" release fully under "Apache License 2.0".
+#        http://www.apache.org/licenses/LICENSE-2.0
 #
-#     This program is free software: you can redistribute it and/or modify
-#     it under the terms of the GNU General Public License as published by
-#     the Free Software Foundation, version 3 of the License.
-#
-#     This program is distributed in the hope that it will be useful,
-#     but WITHOUT ANY WARRANTY; without even the implied warranty of
-#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#     GNU General Public License for more details.
-#
-#     You should have received a copy of the GNU General Public License
-#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#
-#     Please leave the whole of this copyright notice intact.
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 #
 """ Builtins module. Information about builtins of the currently running Python.
 
 """
 
-def isExceptionName( value ):
-    if name.endswith( "Error" ) or name.endswith( "Exception" ):
+def isExceptionName( builtin_name ):
+    if builtin_name.endswith( "Error" ) or builtin_name.endswith( "Exception" ):
         return True
-    elif name in ( "StopIteration", "GeneratorExit" ):
+    elif builtin_name in ( "StopIteration", "GeneratorExit" ):
         return True
     else:
         return False
@@ -52,7 +41,7 @@ try:
     for key in builtin_exception_names:
         builtin_exception_values[ key ] = getattr( exceptions, key )
 
-    del name, key
+    del name, key # Remove names uses in branch, pylint: disable=W0631
 
 except ImportError:
     exceptions = {}
@@ -74,7 +63,7 @@ except ImportError:
     for key, value in exceptions.items():
         builtin_exception_values[ key ] = value
 
-    del name, key, value, x
+    del name, key, value, x # Remove names uses in branch, pylint: disable=W0631
 
 assert "ValueError" in builtin_exception_names
 assert "StopIteration" in builtin_exception_names
@@ -99,6 +88,8 @@ assert "int" in builtin_names
 assert "__doc__" not in builtin_names
 assert "sys" not in builtin_names
 
+builtin_all_names = builtin_names + builtin_exception_names
+
 # For PyLint to be happy.
 assert exceptions
 
