@@ -3,40 +3,29 @@
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
 #
-#     If you submit patches or make the software available to licensors of
-#     this software in either form, you automatically them grant them a
-#     license for your part of the code under "Apache License 2.0" unless you
-#     choose to remove this notice.
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
 #
-#     Kay Hayen uses the right to license his code under only GPL version 3,
-#     to discourage a fork of Nuitka before it is "finished". He will later
-#     make a new "Nuitka" release fully under "Apache License 2.0".
+#        http://www.apache.org/licenses/LICENSE-2.0
 #
-#     This program is free software: you can redistribute it and/or modify
-#     it under the terms of the GNU General Public License as published by
-#     the Free Software Foundation, version 3 of the License.
-#
-#     This program is distributed in the hope that it will be useful,
-#     but WITHOUT ANY WARRANTY; without even the implied warranty of
-#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#     GNU General Public License for more details.
-#
-#     You should have received a copy of the GNU General Public License
-#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#
-#     Please leave the whole of this copyright notice intact.
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 #
 """ Options module """
 
 version_string = """\
-Nuitka V0.3.20.1
+Nuitka V0.3.24
 Copyright (C) 2012 Kay Hayen."""
 
 from . import Utils
 
 from optparse import OptionParser, OptionGroup
 
-import sys, logging
+import os, sys, logging
 
 # Indicator if we were called as "nuitka-python" in which case we assume some other
 # defaults and work a bit different with parameters.
@@ -45,7 +34,7 @@ is_nuitka_python = Utils.basename( sys.argv[0] ).lower() == "nuitka-python"
 def getVersion():
     return version_string.split()[1][1:]
 
-if is_nuitka_python:
+if not is_nuitka_python:
     usage = "usage: %prog [--exe] [--execute] [options] main_module.py"
 else:
     usage = "usage: %prog [options] main_module.py"
@@ -151,7 +140,7 @@ is %s.""" %
 )
 
 execute_group.add_option(
-    "--execute-with-pythonpath",
+    "--execute-with-pythonpath", "--keep-pythonpath",
     action  = "store_true",
     dest    = "keep_pythonpath",
     default = False,
@@ -197,8 +186,9 @@ parser.add_option(
     "--python-version",
     action  = "store",
     dest    = "python_version",
+    choices = ( "2.6", "2.7", "3.2" ),
     default = None,
-    help    = "Major version of Python to be used, something like 2.6 or 2.7."
+    help    = """Major version of Python to be used, one of '2.6', '2.7', or '3.2'."""
 )
 
 parser.add_option(
@@ -328,15 +318,32 @@ Compile the would-be generated source file. Allows edition and translation with
 options for quick debugging changes to the generated source. Defaults to off."""
 )
 
-debug_group.add_option(
-    "--experimental",
-    action  = "store_true",
-    dest    = "experimental",
-    default = False,
-    help    = """\
-Use features declared as "experimental". May have no effect if no experimental features
+def decideExperimental():
+    git_branch_name_filename = os.path.join( os.path.dirname( __file__ ), "..", ".git", "HEAD" )
+
+    if not os.path.exists( git_branch_name_filename ):
+        return False
+
+    branch_name = open( git_branch_name_filename ).read().strip()
+    branch_name = branch_name.split( "/" )[-1]
+
+    if branch_name == "master":
+        return os.path.exists( os.path.join( os.path.dirname( __file__ ), "..", "public-repo" ) )
+    elif branch_name.startswith( "hotfix/" ):
+        return False
+    else:
+        return True
+
+if decideExperimental():
+    debug_group.add_option(
+        "--experimental",
+        action  = "store_true",
+        dest    = "experimental",
+        default = False,
+        help    = """\
+Use features declared as 'experimental'. May have no effect if no experimental features
 are present in the code. Defaults to off."""
-)
+    )
 
 parser.add_option_group( debug_group )
 
@@ -438,9 +445,6 @@ if options.recurse_all and not options.executable:
 if options.recurse_stdlib and not options.executable:
     sys.exit( "Error, options '--recurse-stdlib' makes no sense without option '--exe'." )
 
-def useValuePropagation():
-    return options.experimental
-
 def shallTraceExecution():
     return options.trace_execution
 
@@ -539,3 +543,9 @@ def isShowProgress():
 
 def isRemoveBuildDir():
     return options.remove_build
+
+def getIntendedPythonVersion():
+    return options.python_version
+
+def isExperimental():
+    return hasattr( options, "experimental" ) and options.experimental
