@@ -3,28 +3,17 @@
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
 #
-#     If you submit patches or make the software available to licensors of
-#     this software in either form, you automatically them grant them a
-#     license for your part of the code under "Apache License 2.0" unless you
-#     choose to remove this notice.
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
 #
-#     Kay Hayen uses the right to license his code under only GPL version 3,
-#     to discourage a fork of Nuitka before it is "finished". He will later
-#     make a new "Nuitka" release fully under "Apache License 2.0".
+#        http://www.apache.org/licenses/LICENSE-2.0
 #
-#     This program is free software: you can redistribute it and/or modify
-#     it under the terms of the GNU General Public License as published by
-#     the Free Software Foundation, version 3 of the License.
-#
-#     This program is distributed in the hope that it will be useful,
-#     but WITHOUT ANY WARRANTY; without even the implied warranty of
-#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#     GNU General Public License for more details.
-#
-#     You should have received a copy of the GNU General Public License
-#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#
-#     Please leave the whole of this copyright notice intact.
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 #
 """ Source code reference record.
 
@@ -50,19 +39,28 @@ class SourceCodeReference:
         self.future_spec = None
         self.inside_exec = False
 
+        self.set_line = True
+
     def __repr__( self ):
-        return "<SourceCodeReference to %s:%s>" % ( self.filename, self.line )
+        return "<%s to %s:%s>" % ( self.__class__.__name__, self.filename, self.line )
 
-    def atLineNumber( self, line ):
-        assert int( line ) == line
-
-        return SourceCodeReference.fromFilenameAndLine(
+    def clone( self, line ):
+        result = SourceCodeReference.fromFilenameAndLine(
             filename    = self.filename,
             line        = line,
             future_spec = self.future_spec,
             inside_exec = self.inside_exec
         )
 
+        result.set_line = self.set_line
+
+        return result
+
+    def atLineNumber( self, line ):
+        assert int( line ) == line
+
+        return self.clone( line )
+
     def getLineNumber( self ):
         return self.line
 
@@ -76,7 +74,7 @@ class SourceCodeReference:
         return "%s:%s" % ( self.filename, self.line )
 
     def getExecReference( self ):
-        return SourceCodeReference.fromFilenameAndLine(
+        return self.__class__.fromFilenameAndLine(
             filename    = self.filename,
             line        = self.line,
             future_spec = self.future_spec.clone(),
@@ -86,6 +84,9 @@ class SourceCodeReference:
     def isExecReference( self ):
         return self.inside_exec
 
+    def shallSetCurrentLine( self ):
+        return self.set_line
+
     def __cmp__( self, other ):
         if other is None:
             return -1
@@ -99,6 +100,16 @@ class SourceCodeReference:
 
         return result
 
+    def atInternal( self ):
+        if self.set_line:
+            result = self.clone( self.line )
+            result.set_line = False
+
+            return result
+        else:
+            return self
+
+
 def fromFilename( filename, future_spec ):
     return SourceCodeReference.fromFilenameAndLine(
         filename    = filename,
