@@ -3,28 +3,17 @@
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
 #
-#     If you submit patches or make the software available to licensors of
-#     this software in either form, you automatically them grant them a
-#     license for your part of the code under "Apache License 2.0" unless you
-#     choose to remove this notice.
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
 #
-#     Kay Hayen uses the right to license his code under only GPL version 3,
-#     to discourage a fork of Nuitka before it is "finished". He will later
-#     make a new "Nuitka" release fully under "Apache License 2.0".
+#        http://www.apache.org/licenses/LICENSE-2.0
 #
-#     This program is free software: you can redistribute it and/or modify
-#     it under the terms of the GNU General Public License as published by
-#     the Free Software Foundation, version 3 of the License.
-#
-#     This program is distributed in the hope that it will be useful,
-#     but WITHOUT ANY WARRANTY; without even the implied warranty of
-#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#     GNU General Public License for more details.
-#
-#     You should have received a copy of the GNU General Public License
-#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#
-#     Please leave the whole of this copyright notice intact.
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 #
 """ Code templates one stop access. """
 
@@ -35,9 +24,7 @@ from .templates.CodeTemplatesMain import *
 from .templates.CodeTemplatesConstants import *
 
 from .templates.CodeTemplatesFunction import *
-from .templates.CodeTemplatesGeneratorExpression import *
 from .templates.CodeTemplatesGeneratorFunction import *
-from .templates.CodeTemplatesContraction import *
 
 from .templates.CodeTemplatesParameterParsing import *
 
@@ -48,9 +35,7 @@ from .templates.CodeTemplatesTuples import *
 from .templates.CodeTemplatesLists import *
 from .templates.CodeTemplatesDicts import *
 from .templates.CodeTemplatesCalls import *
-from .templates.CodeTemplatesClass import *
 from .templates.CodeTemplatesLoops import *
-from .templates.CodeTemplatesWith import *
 
 from .templates.CodeTemplatesExecEval import *
 
@@ -130,3 +115,46 @@ else
 {
 %(branch_no_code)s
 }"""
+
+def enableDebug():
+    templates = dict( globals() )
+
+    class TemplateWrapper:
+        """ Wrapper around templates, to better trace and control template usage. """
+        def __init__( self, name, value ):
+            self.name = name
+            self.value = value
+
+        def __str__( self ):
+            return self.value
+
+        def __mod__( self, other ):
+            assert type( other ) is dict, self.name
+
+            for key in other.keys():
+                if "%%(%s)" % key not in self.value:
+                    from logging import warning
+
+                    warning( "Extra value '%s' provided to template '%s'.", key, self.name )
+
+            return self.value % other
+
+        def split( self, sep ):
+            return self.value.split( sep )
+
+    from nuitka.__past__ import iterItems
+
+    for template_name, template_value in iterItems( templates ):
+        if template_name.startswith( "_" ):
+            continue
+
+        if type( template_value ) is str:
+            globals()[ template_name ] = TemplateWrapper(
+                template_name,
+                template_value
+            )
+
+from nuitka.Options import isDebug
+
+if isDebug():
+    enableDebug()
