@@ -3,28 +3,17 @@
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
 #
-#     If you submit patches or make the software available to licensors of
-#     this software in either form, you automatically them grant them a
-#     license for your part of the code under "Apache License 2.0" unless you
-#     choose to remove this notice.
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
 #
-#     Kay Hayen uses the right to license his code under only GPL version 3,
-#     to discourage a fork of Nuitka before it is "finished". He will later
-#     make a new "Nuitka" release fully under "Apache License 2.0".
+#        http://www.apache.org/licenses/LICENSE-2.0
 #
-#     This program is free software: you can redistribute it and/or modify
-#     it under the terms of the GNU General Public License as published by
-#     the Free Software Foundation, version 3 of the License.
-#
-#     This program is distributed in the hope that it will be useful,
-#     but WITHOUT ANY WARRANTY; without even the implied warranty of
-#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#     GNU General Public License for more details.
-#
-#     You should have received a copy of the GNU General Public License
-#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#
-#     Please leave the whole of this copyright notice intact.
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 #
 """ Code generation contexts.
 
@@ -41,7 +30,31 @@ from .Identifiers import (
 
 from .Namify import namifyConstant
 
-from ..Constants import HashableConstant
+from nuitka.Constants import HashableConstant
+
+from nuitka.__past__ import iterItems
+
+from nuitka.Utils import python_version
+
+# False alarms about "hashlib.md5" due to its strange way of defining what is
+# exported, pylint won't understand it. pylint: disable=E1101
+
+import hashlib
+
+if python_version < 300:
+    def calcHash( key ):
+        hash_value = hashlib.md5(
+            "%s%s%d%s%s" % key
+        )
+
+        return hash_value.hexdigest()
+else:
+    def calcHash( key ):
+        hash_value = hashlib.md5(
+            ( "%s%s%d%s%s" % key ).encode( "utf-8" )
+        )
+
+        return hash_value.hexdigest()
 
 # Many methods won't use self, but it's the interface. pylint: disable=R0201
 
@@ -60,27 +73,11 @@ class PythonContextBase:
 
         return self.for_loop_count
 
-    def allocateWhileLoopNumber( self ):
-        self.while_loop_count += 1
-
-        return self.while_loop_count
-
     def allocateTryNumber( self ):
         self.try_count += 1
 
         return self.try_count
 
-    def allocateWithNumber( self ):
-        self.with_count += 1
-
-        return self.with_count
-
-    def isClosureViaContext( self ):
-        return True
-
-    def isParametersViaContext( self ):
-        return False
-
     def hasLocalsDict( self ):
         return False
 
@@ -88,7 +85,7 @@ class PythonContextBase:
         return False
 
     def getTempObjectHandle( self, var_name ):
-        return TempObjectIdentifier( var_name, from_context = "" )
+        return TempObjectIdentifier( var_name )
 
     def getTempVarHandle( self, var_name ):
         return TempVariableIdentifier( var_name )
@@ -121,12 +118,6 @@ class PythonChildContextBase( PythonContextBase ):
     def addFunctionCodes( self, code_name, function_decl, function_code ):
         self.parent.addFunctionCodes( code_name, function_decl, function_code )
 
-    def addContractionCodes( self, code_name, contraction_decl, contraction_code ):
-        self.parent.addContractionCodes( code_name, contraction_decl, contraction_code )
-
-    def addClassCodes( self, code_name, class_decl, class_code ):
-        self.parent.addClassCodes( code_name, class_decl, class_code )
-
     def addGlobalVariableNameUsage( self, var_name ):
         self.parent.addGlobalVariableNameUsage( var_name )
 
@@ -136,6 +127,9 @@ class PythonChildContextBase( PythonContextBase ):
     def getModuleName( self ):
         return self.parent.getModuleName()
 
+    def getCodeObjectHandle( self, filename, code_name, line_number, arg_names, is_generator ):
+        return self.parent.getCodeObjectHandle( filename, code_name, line_number, arg_names, is_generator )
+
 class PythonGlobalContext:
     def __init__( self ):
         self.constants = {}
@@ -153,6 +147,7 @@ class PythonGlobalContext:
         # Python mechanics.
         self.getConstantHandle( "__module__" )
         self.getConstantHandle( "__class__" )
+        self.getConstantHandle( "__metaclass__" )
         self.getConstantHandle( "__dict__" )
         self.getConstantHandle( "__doc__" )
         self.getConstantHandle( "__file__" )
@@ -180,14 +175,26 @@ class PythonGlobalContext:
         self.getConstantHandle( "read" )
         self.getConstantHandle( "strip" )
 
+        # Executables only
+        self.getConstantHandle( "__main__" )
+
+        # sys exception attributes
+        self.getConstantHandle( "exc_type" )
+        self.getConstantHandle( "exc_value" )
+        self.getConstantHandle( "exc_traceback" )
+
         # Have EVAL_ORDER for 1..6 in any case, so we can use it in the C++ code freely
         # without concern.
         self.make_tuples_used = set( range( 1, 6 ) )
         self.make_lists_used = set( range( 0, 1 ) )
         self.make_dicts_used = set( range( 0, 3 ) )
 
+        # Include 0, so list and tuple making can use it without being special.
         self.eval_orders_used = set( range( 0, 6 ) )
 
+        # Code objects needed.
+        self.code_objects = {}
+
     def getConstantHandle( self, constant ):
         if constant is None:
             return Identifier( "Py_None", 0 )
@@ -208,6 +215,24 @@ class PythonGlobalContext:
     def getConstants( self ):
         return sorted( self.constants.items(), key = lambda x: x[1] )
 
+    def getCodeObjectHandle( self, filename, code_name, line_number, arg_names, is_generator ):
+        key = ( filename, code_name, line_number, arg_names, is_generator )
+
+        if key not in self.code_objects:
+            self.code_objects[ key ] = Identifier(
+                "_codeobj_%s" % calcHash( key ),
+                0
+            )
+
+            self.getConstantHandle( filename )
+            self.getConstantHandle( code_name )
+            self.getConstantHandle( arg_names )
+
+        return self.code_objects[ key ]
+
+    def getCodeObjects( self ):
+        return sorted( iterItems( self.code_objects ) )
+
     def addEvalOrderUse( self, value ):
         assert type( value ) is int
 
@@ -257,9 +282,7 @@ class PythonModuleContext( PythonContextBase ):
 
         self.global_context = global_context
 
-        self.class_codes = {}
         self.function_codes = {}
-        self.contraction_codes = {}
 
         self.global_var_names = set()
         self.temp_var_names = set()
@@ -279,6 +302,9 @@ class PythonModuleContext( PythonContextBase ):
     def getConstantHandle( self, constant ):
         return self.global_context.getConstantHandle( constant )
 
+    def getCodeObjectHandle( self, filename, code_name, line_number, arg_names, is_generator ):
+        return self.global_context.getCodeObjectHandle( filename, code_name, line_number, arg_names, is_generator )
+
     def addFunctionCodes( self, code_name, function_decl, function_code ):
         assert code_name not in self.function_codes
 
@@ -287,29 +313,13 @@ class PythonModuleContext( PythonContextBase ):
     def getFunctionsCodes( self ):
         return self.function_codes
 
-    def addContractionCodes( self, code_name, contraction_decl, contraction_code ):
-        assert code_name not in self.contraction_codes
-
-        self.contraction_codes[ code_name ] = ( contraction_decl, contraction_code )
-
-    def getContractionsCodes( self ):
-        return self.contraction_codes
-
-    def addClassCodes( self, code_name, class_decl, class_code ):
-        assert code_name not in self.class_codes
-
-        self.class_codes[ code_name ] = ( class_decl, class_code )
-
-    def getClassesCodes( self ):
-        return self.class_codes
-
     def getName( self ):
         return self.name
 
     getModuleName = getName
 
     def getModuleCodeName( self ):
-        return self.getCodeName()
+        return self.code_name
 
     # There cannot ne local variable in modules no need to consider the name.
     # pylint: disable=W0613
@@ -320,24 +330,12 @@ class PythonModuleContext( PythonContextBase ):
         return False
     # pylint: enable=W0613
 
-    def canHaveLocalVariables( self ):
-        return False
-
     def addGlobalVariableNameUsage( self, var_name ):
         self.global_var_names.add( var_name )
 
     def getGlobalVariableNames( self ):
         return self.global_var_names
 
-    def getCodeName( self ):
-        return self.code_name
-
-    def getTracebackName( self ):
-        return "<module>"
-
-    def getTracebackFilename( self ):
-        return self.filename
-
     def addEvalOrderUse( self, value ):
         self.global_context.addEvalOrderUse( value )
 
@@ -357,12 +355,17 @@ class PythonFunctionContext( PythonChildContextBase ):
 
         self.function = function
 
+        self.needs_creation = function.needsCreation()
+
         # Make sure the local names are available as constants
         for local_name in function.getLocalVariableNames():
             self.getConstantHandle( constant = local_name )
 
     def __repr__( self ):
-        return "<PythonFunctionContext for function '%s'>" % self.function.getName()
+        return "<PythonFunctionContext for %s '%s'>" % (
+            "function" if not self.function.isClassDictCreation() else "class",
+            self.function.getName()
+        )
 
     def hasLocalsDict( self ):
         return self.function.hasLocalsDict()
@@ -373,12 +376,6 @@ class PythonFunctionContext( PythonChildContextBase ):
     def hasClosureVariable( self, var_name ):
         return var_name in self.function.getClosureVariableNames()
 
-    def canHaveLocalVariables( self ):
-        return True
-
-    def isParametersViaContext( self ):
-        return self.function.isGenerator()
-
     def getFrameHandle( self ):
         if self.function.isGenerator():
             return Identifier( "generator->m_frame", 0 )
@@ -392,169 +389,28 @@ class PythonFunctionContext( PythonChildContextBase ):
         return LocalVariableIdentifier( var_name, from_context = self.function.isGenerator() )
 
     def getClosureHandle( self, var_name ):
-        if not self.function.isGenerator():
-            return ClosureVariableIdentifier( var_name, from_context = "_python_context->" )
+        if self.needs_creation:
+            if self.function.isGenerator():
+                return ClosureVariableIdentifier( var_name, from_context = "_python_context->common_context->" )
+            else:
+                return ClosureVariableIdentifier( var_name, from_context = "_python_context->" )
         else:
-            return ClosureVariableIdentifier( var_name, from_context = "_python_context->common_context->" )
+            if self.function.isGenerator():
+                return ClosureVariableIdentifier( var_name, from_context = "_python_context->" )
+            else:
+                return ClosureVariableIdentifier( var_name, from_context = "" )
 
     def getTempObjectHandle( self, var_name ):
-        if self.function.isGenerator():
-            return TempObjectIdentifier( var_name, from_context = "_python_context->" )
-        else:
-            return TempObjectIdentifier( var_name, from_context = "" )
-
-    def getCodeName( self ):
-        return self.function.getCodeName()
-
-    def getTracebackName( self ):
-        return self.function.getName()
-
-    # TODO: Memoize would do wonders for these things mayhaps.
-    def getTracebackFilename( self ):
-        return self.function.getParentModule().getFilename()
+        return TempObjectIdentifier( var_name )
 
     def needsFrameExceptionKeeper( self ):
         return self.function.needsFrameExceptionKeeper()
 
-class PythonContractionBase( PythonChildContextBase ):
-    def __init__( self, parent, contraction ):
-        PythonChildContextBase.__init__( self, parent = parent )
-
-        self.contraction = contraction
-
-    def isClosureViaContext( self ):
-        return False
-
-    def isGeneratorExpression( self ):
-        return self.__class__ == PythonGeneratorExpressionContext
-
-    def getTracebackFilename( self ):
-        return self.contraction.getParentModule().getFilename()
-
-    def getTracebackName( self ):
-        return self.contraction.getName()
-
-class PythonListContractionContext( PythonContractionBase ):
-    def __init__( self, parent, contraction ):
-        PythonContractionBase.__init__(
-            self,
-            parent      = parent,
-            contraction = contraction
-        )
-
-    def getClosureHandle( self, var_name ):
-        return ClosureVariableIdentifier( var_name, from_context = "" )
-
-    def getLocalHandle( self, var_name ):
-        return self.getClosureHandle( var_name )
-
-    def hasFrameGuard( self ):
-        return True
-
-class PythonGeneratorExpressionContext( PythonContractionBase ):
-    def __init__( self, parent, contraction ):
-        PythonContractionBase.__init__(
-            self,
-            parent      = parent,
-            contraction = contraction
-        )
-
-    def getClosureHandle( self, var_name ):
-        return ClosureVariableIdentifier( var_name, from_context = "_python_context->" )
-
-    def getLocalHandle( self, var_name ):
-        return LocalVariableIdentifier( var_name, from_context = True )
-
-    def getTempObjectHandle( self, var_name ):
-        return TempObjectIdentifier( var_name, from_context = "_python_context->" )
-
-    def hasFrameGuard( self ):
-        return False
-
-class PythonSetContractionContext( PythonContractionBase ):
-    def __init__( self, parent, contraction ):
-        PythonContractionBase.__init__(
-            self,
-            parent      = parent,
-            contraction = contraction
-        )
-
-    def getClosureHandle( self, var_name ):
-        return ClosureVariableIdentifier( var_name, from_context = "" )
-
-    def getLocalHandle( self, var_name ):
-        return LocalVariableIdentifier( var_name, from_context = False )
-
-    def hasFrameGuard( self ):
-        return True
-
-class PythonDictContractionContext( PythonContractionBase ):
-    def __init__( self, parent, contraction ):
-        PythonContractionBase.__init__(
-            self,
-            parent      = parent,
-            contraction = contraction
-        )
-
-    def getClosureHandle( self, var_name ):
-        return ClosureVariableIdentifier( var_name, from_context = "" )
-
-    def getLocalHandle( self, var_name ):
-        return LocalVariableIdentifier( var_name, from_context = False )
-
-    def hasFrameGuard( self ):
-        return True
-
-class PythonClassContext( PythonChildContextBase ):
-    def __init__( self, parent, class_def ):
-        PythonChildContextBase.__init__( self, parent = parent )
-
-        self.class_def = class_def
-
-    def hasLocalVariable( self, var_name ):
-        return var_name in self.class_def.getClassVariableNames()
-
-    def hasClosureVariable( self, var_name ):
-        return var_name in self.class_def.getClosureVariableNames()
-
-    def getFrameHandle( self ):
-        return Identifier( "frame_guard.getFrame()", 1 )
-
-    def hasFrameGuard( self ):
-        return True
-
-    def getLocalHandle( self, var_name ):
-        return LocalVariableIdentifier( var_name )
-
-    def getClosureHandle( self, var_name ):
-        return ClosureVariableIdentifier( var_name, from_context = "" )
-
-    def isClosureViaContext( self ):
-        return False
-
-    def getTracebackName( self ):
-        return self.class_def.getName()
-
-    def getTracebackFilename( self ):
-        return self.class_def.getParentModule().getFilename()
-
-    def getCodeName( self ):
-        return self.class_def.getCodeName()
-
-    def hasLocalsDict( self ):
-        return self.class_def.hasLocalsDict()
-
-    def needsFrameExceptionKeeper( self ):
-        return self.class_def.needsFrameExceptionKeeper()
-
 
 class PythonExecInlineContext( PythonChildContextBase ):
     def __init__( self, parent ):
         PythonChildContextBase.__init__( self, parent = parent )
 
-    def canHaveLocalVariables( self ):
-        return self.parent.canHaveLocalVariables()
-
     def getClosureHandle( self, var_name ):
         return self.parent.getLocalHandle( var_name )
 
