@@ -3,28 +3,17 @@
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
 #
-#     If you submit patches or make the software available to licensors of
-#     this software in either form, you automatically them grant them a
-#     license for your part of the code under "Apache License 2.0" unless you
-#     choose to remove this notice.
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
 #
-#     Kay Hayen uses the right to license his code under only GPL version 3,
-#     to discourage a fork of Nuitka before it is "finished". He will later
-#     make a new "Nuitka" release fully under "Apache License 2.0".
+#        http://www.apache.org/licenses/LICENSE-2.0
 #
-#     This program is free software: you can redistribute it and/or modify
-#     it under the terms of the GNU General Public License as published by
-#     the Free Software Foundation, version 3 of the License.
-#
-#     This program is distributed in the hope that it will be useful,
-#     but WITHOUT ANY WARRANTY; without even the implied warranty of
-#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#     GNU General Public License for more details.
-#
-#     You should have received a copy of the GNU General Public License
-#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#
-#     Please leave the whole of this copyright notice intact.
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 #
 """ Generator function (with yield) related templates.
 
@@ -55,11 +44,12 @@ static void _context_common_%(function_identifier)s_destructor( void *context_vo
 {
     _context_common_%(function_identifier)s_t *_python_context = (struct _context_common_%(function_identifier)s_t *)context_voidptr;
 
+    assert( _python_context->ref_count > 0 );
     _python_context->ref_count -= 1;
+%(context_free)s
 
     if ( _python_context->ref_count == 0 )
     {
-%(context_free)s
         delete _python_context;
     }
 }
@@ -74,6 +64,21 @@ static void _context_generator_%(function_identifier)s_destructor( void *context
 }
 """
 
+genfunc_context_local_only_template = """
+struct _context_generator_%(function_identifier)s_t
+{
+    // The generator function instance can access its parameters from creation time.
+%(function_instance_context_decl)s
+};
+
+static void _context_generator_%(function_identifier)s_destructor( void *context_voidptr )
+{
+    _context_generator_%(function_identifier)s_t *_python_context = (struct _context_generator_%(function_identifier)s_t *)context_voidptr;
+
+    delete _python_context;
+}
+"""
+
 make_genfunc_with_context_template = """
 static PyObject *_MAKE_FUNCTION_%(function_identifier)s( %(function_creation_args)s )
 {
@@ -83,65 +88,40 @@ static PyObject *_MAKE_FUNCTION_%(function_identifier)s( %(function_creation_arg
     // Copy the parameter default values and closure values over.
 %(context_copy)s
 
-    PyObject *result = Nuitka_Function_New(
+    return Nuitka_Function_New(
         %(fparse_function_identifier)s,
         %(mparse_function_identifier)s,
         %(function_name_obj)s,
-        _CODEOBJ_%(function_identifier)s ? _CODEOBJ_%(function_identifier)s : ( _CODEOBJ_%(function_identifier)s = MAKE_CODEOBJ( %(filename_identifier)s, %(function_name_obj)s, %(line_number)d, %(arg_names)s, %(arg_count)d, true ) ),
+        %(code_identifier)s,
         %(defaults)s,
         %(module_identifier)s,
         %(function_doc)s,
         _python_context,
         _context_common_%(function_identifier)s_destructor
     );
-
-    return result;
 }
+"""
 
+make_genfunc_without_context_template = """
+static PyObject *_MAKE_FUNCTION_%(function_identifier)s( %(function_creation_args)s )
+{
+    return Nuitka_Function_New(
+        %(fparse_function_identifier)s,
+        %(mparse_function_identifier)s,
+        %(function_name_obj)s,
+        %(code_identifier)s,
+        %(defaults)s,
+        %(module_identifier)s,
+        %(function_doc)s
+    );
+}
 """
 
 genfunc_yielder_template = """
-static PyFrameObject *frame_%(function_identifier)s = NULL;
-static PyCodeObject *_CODEOBJ_%(function_identifier)s = NULL;
-
 static void %(function_identifier)s_context( Nuitka_GeneratorObject *generator )
 {
-    bool traceback;
-
-    // Must be inside block, or else its d-tor will not be run.
-    if ( isFrameUnusable( frame_%(function_identifier)s ) )
-    {
-        if ( frame_%(function_identifier)s )
-        {
-#if _DEBUG_REFRAME
-            puts( "reframe for %(function_identifier)s" );
-#endif
-            Py_DECREF( frame_%(function_identifier)s );
-        }
-
-        assertCodeObject( _CODEOBJ_%(function_identifier)s );
-
-        frame_%(function_identifier)s = MAKE_FRAME( _CODEOBJ_%(function_identifier)s, %(module_identifier)s );
-    }
-
-    Py_INCREF( frame_%(function_identifier)s );
-    generator->m_frame = frame_%(function_identifier)s;
-
-    Py_CLEAR( generator->m_frame->f_back );
-
-    generator->m_frame->f_back = PyThreadState_GET()->frame;
-    Py_INCREF( generator->m_frame->f_back );
-
-    PyThreadState_GET()->frame = generator->m_frame;
-
-    FrameGuardLight frame_guard( &generator->m_frame );
-
-    try
-    {
-        traceback = true;
-        CHECK_EXCEPTION( generator );
-        traceback = false;
-
+    try {
+        // Make context accessible if one is used.
 %(context_access)s
 
         // Local variable inits
@@ -149,59 +129,121 @@ static void %(function_identifier)s_context( Nuitka_GeneratorObject *generator )
 
         // Actual function code.
 %(function_body)s
-
-        PyErr_SetNone( PyExc_StopIteration );
-        generator->m_yielded = NULL;
     }
     catch ( ReturnException &e )
     {
         PyErr_SetNone( PyExc_StopIteration );
-        generator->m_yielded = NULL;
-    }
-    catch ( _PythonException &_exception )
-    {
-        if ( traceback == false )
-        {
-           _exception.addTraceback( INCREASE_REFCOUNT( generator->m_frame ) );
-        }
-
-        _exception.toPython();
-
-        generator->m_yielded = NULL;
     }
 
+    // TODO: Won't return, we should tell the compiler about that.
+    generator->m_yielded = NULL;
     swapFiber( &generator->m_yielder_context, &generator->m_caller_context );
 }
 """
 
-genfunc_function_template = """
-static PyObject *impl_%(function_identifier)s( PyObject *self%(parameter_objects_decl)s )
+frame_guard_genfunc_template = """\
+static PyFrameObject *frame_%(frame_identifier)s = NULL;
+
+// Must be inside block, or else its d-tor will not be run.
+if ( isFrameUnusable( frame_%(frame_identifier)s ) )
 {
-    struct _context_common_%(function_identifier)s_t *_python_common_context = (struct _context_common_%(function_identifier)s_t *)self;
-
-    try
+    if ( frame_%(frame_identifier)s )
     {
-        struct _context_generator_%(function_identifier)s_t *_python_context = new _context_generator_%(function_identifier)s_t;
+#if _DEBUG_REFRAME
+        puts( "reframe for %(frame_identifier)s" );
+#endif
+        Py_DECREF( frame_%(frame_identifier)s );
+    }
 
-        _python_context->common_context = _python_common_context;
-        _python_common_context->ref_count += 1;
+    frame_%(frame_identifier)s = MAKE_FRAME( %(code_identifier)s, %(module_identifier)s );
+}
 
+Py_INCREF( frame_%(frame_identifier)s );
+generator->m_frame = frame_%(frame_identifier)s;
+
+Py_CLEAR( generator->m_frame->f_back );
+
+generator->m_frame->f_back = PyThreadState_GET()->frame;
+Py_INCREF( generator->m_frame->f_back );
+
+PyThreadState_GET()->frame = generator->m_frame;
+
+FrameGuardLight frame_guard( &generator->m_frame );
+
+bool traceback;
+
+try
+{
+    // TODO: In case we don't raise exceptions ourselves, we would still have to do this, so
+    // beware to not optimize this away for generators without a replacement.
+    traceback = true;
+    CHECK_EXCEPTION( generator );
+    traceback = false;
+
+%(codes)s
+
+    PyErr_SetNone( PyExc_StopIteration );
+}
+catch ( _PythonException &_exception )
+{
+    if ( traceback == false )
+    {
+       _exception.addTraceback( INCREASE_REFCOUNT( generator->m_frame ) );
+    }
+
+    _exception.toPython();
+
+    // TODO: Moving this code is not allowed yet.
+    generator->m_yielded = NULL;
+}"""
+
+genfunc_common_context_use_template = """\
+struct _context_common_%(function_identifier)s_t *_python_common_context = (struct _context_common_%(function_identifier)s_t *)self;
+struct _context_generator_%(function_identifier)s_t *_python_context = new _context_generator_%(function_identifier)s_t;
+
+_python_context->common_context = _python_common_context;
+_python_common_context->ref_count += 1;"""
+
+genfunc_local_context_use_template = """\
+struct _context_generator_%(function_identifier)s_t *_python_context = \
+new _context_generator_%(function_identifier)s_t;"""
+
+
+genfunc_generator_without_context_making = """\
         PyObject *result = Nuitka_Generator_New(
             %(function_identifier)s_context,
             %(function_name_obj)s,
+            %(code_identifier)s
+        );"""
+
+genfunc_generator_with_context_making = """\
+        PyObject *result = Nuitka_Generator_New(
+            %(function_identifier)s_context,
+            %(function_name_obj)s,
+            %(code_identifier)s,
             _python_context,
             _context_generator_%(function_identifier)s_destructor
-        );
+        );"""
+
+
+genfunc_function_maker_template = """
+static PyObject *impl_%(function_identifier)s( %(parameter_objects_decl)s )
+{
+    // Create context if any
+%(context_making)s
+
+    try
+    {
+%(generator_making)s
 
         if (unlikely( result == NULL ))
         {
-            delete _python_context;
-
             PyErr_Format( PyExc_RuntimeError, "cannot create function %(function_name)s" );
-            throw _PythonException();
+            return NULL;
         }
 
-%(parameter_context_assign)s
+        // Copy to context parameter values and closured variables if any.
+%(context_copy)s
 
         return result;
     }
@@ -212,8 +254,6 @@ static PyObject *impl_%(function_identifier)s( PyObject *self%(parameter_objects
         return NULL;
     }
 }
-
-%(parameter_entry_point_code)s
 """
 
 generator_context_access_template = """
