@@ -3,28 +3,17 @@
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
 #
-#     If you submit patches or make the software available to licensors of
-#     this software in either form, you automatically them grant them a
-#     license for your part of the code under "Apache License 2.0" unless you
-#     choose to remove this notice.
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
 #
-#     Kay Hayen uses the right to license his code under only GPL version 3,
-#     to discourage a fork of Nuitka before it is "finished". He will later
-#     make a new "Nuitka" release fully under "Apache License 2.0".
+#        http://www.apache.org/licenses/LICENSE-2.0
 #
-#     This program is free software: you can redistribute it and/or modify
-#     it under the terms of the GNU General Public License as published by
-#     the Free Software Foundation, version 3 of the License.
-#
-#     This program is distributed in the hope that it will be useful,
-#     but WITHOUT ANY WARRANTY; without even the implied warranty of
-#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#     GNU General Public License for more details.
-#
-#     You should have received a copy of the GNU General Public License
-#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#
-#     Please leave the whole of this copyright notice intact.
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 #
 """ Main module code templates
 
@@ -34,26 +23,25 @@ global_copyright = """\
 // Generated code for Python source for module '%(name)s'
 // created by Nuitka version %(version)s
 
-// This code is in part copyright Kay Hayen, license GPLv3. This has the
-// consequence that your must either obtain another license or also publish
-// your original source code under a compatible license unless you don't
-// distribute this source or the compiled binary.
+// This code is in part copyright 2012 Kay Hayen.
+//
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
 """
 
 module_inittab_entry = """\
-{ (char *)"%(module_name)s", init%(module_identifier)s },"""
+{ (char *)"%(module_name)s", MOD_INIT_NAME( %(module_identifier)s ) },"""
 
 main_program = """\
-// Our own inittab for lookup of "frozen" modules, i.e. the ones included in this binary.
-static struct _inittab _frozes_modules[] =
-{
-%(module_inittab)s
-    { NULL, NULL }
-};
-
-// For embedded modules, to be unpacked. Used by main program only
-extern void registerMetaPathBasedUnfreezer( struct _inittab *_frozes_modules );
-
 // The main program for C++. It needs to prepare the interpreter and then calls the
 // initialization code of the __main__ module.
 
@@ -69,7 +57,6 @@ int main( int argc, char *argv[] )
     PyType_Ready( &Nuitka_Generator_Type );
     PyType_Ready( &Nuitka_Function_Type );
     PyType_Ready( &Nuitka_Method_Type );
-    PyType_Ready( &Nuitka_Genexpr_Type );
 
     enhancePythonTypes();
 
@@ -77,28 +64,15 @@ int main( int argc, char *argv[] )
     // or to python.exe on Windows.
     PySys_SetObject(
         (char *)"executable",
-#if PYTHON_VERSION < 300
-        PyString_FromString( %(sys_executable)s )
-#else
-        PyUnicode_FromString( %(sys_executable)s )
-#endif
+        %(sys_executable)s
     );
 
-    // Register the initialization functions for modules included in the binary if any
-    int res = PyImport_ExtendInittab( _frozes_modules );
-    assert( res != -1 );
-
-    registerMetaPathBasedUnfreezer( _frozes_modules );
-
     patchInspectModule();
 
-#if PYTHON_VERSION < 300
-    init__main__();
-#else
-    PyInit___main__();
-#endif
+    // Execute the "__main__" module init function.
+    MOD_INIT_NAME( __main__)();
 
-    if ( PyErr_Occurred() )
+    if ( ERROR_OCCURED() )
     {
         assertFrameObject( frame___main__ );
         assert( frame___main__->f_back == NULL );
@@ -121,7 +95,7 @@ module_header_template = """\
 
 #include <nuitka/helpers.hpp>
 
-MOD_INIT( %(module_identifier)s );
+MOD_INIT_DECL( %(module_identifier)s );
 
 extern PyObject *_module_%(module_identifier)s;
 
@@ -146,7 +120,7 @@ class PyObjectGlobalVariable_%(module_identifier)s
                 return entry->me_value;
             }
 
-            entry = GET_PYDICT_ENTRY( _module_builtin, *this->var_name );
+            entry = GET_PYDICT_ENTRY( module_builtin, *this->var_name );
 
             if (likely( entry->me_value != NULL ))
             {
@@ -178,7 +152,26 @@ class PyObjectGlobalVariable_%(module_identifier)s
             }
         }
 
-        void assign( PyObject *value ) const
+        void assign0( PyObject *value ) const
+        {
+            PyDictEntry *entry = GET_PYDICT_ENTRY( (PyModuleObject *)_module_%(module_identifier)s, *this->var_name );
+
+            // Values are more likely set than not set, in that case speculatively try the
+            // quickest access method.
+            if (likely( entry->me_value != NULL ))
+            {
+                PyObject *old = entry->me_value;
+                entry->me_value = INCREASE_REFCOUNT( value );
+
+                Py_DECREF( old );
+            }
+            else
+            {
+                DICT_SET_ITEM( ((PyModuleObject *)_module_%(module_identifier)s)->md_dict, (PyObject *)*this->var_name, value );
+            }
+        }
+
+        void assign1( PyObject *value ) const
         {
             PyDictEntry *entry = GET_PYDICT_ENTRY( (PyModuleObject *)_module_%(module_identifier)s, *this->var_name );
 
@@ -199,32 +192,13 @@ class PyObjectGlobalVariable_%(module_identifier)s
             }
         }
 
-        void assign0( PyObject *value ) const
-        {
-            PyDictEntry *entry = GET_PYDICT_ENTRY( (PyModuleObject *)_module_%(module_identifier)s, *this->var_name );
-
-            // Values are more likely set than not set, in that case speculatively try the
-            // quickest access method.
-            if (likely( entry->me_value != NULL ))
-            {
-                PyObject *old = entry->me_value;
-                entry->me_value = INCREASE_REFCOUNT( value );
-
-                Py_DECREF( old );
-            }
-            else
-            {
-                DICT_SET_ITEM( ((PyModuleObject *)_module_%(module_identifier)s)->md_dict, (PyObject *)*this->var_name, value );
-            }
-        }
-
         void del() const
         {
             int status = PyDict_DelItem( ((PyModuleObject *)_module_%(module_identifier)s)->md_dict, (PyObject *)*this->var_name );
 
             if (unlikely( status == -1 ))
             {
-                PyErr_Format( PyExc_NameError, "name '%%s' is not defined", Nuitka_String_AsString( (PyObject *)*this->var_name ) );
+                PyErr_Format( PyExc_NameError, "global name '%%s' is not defined", Nuitka_String_AsString( (PyObject *)*this->var_name ) );
                 throw _PythonException();
             }
         }
@@ -240,7 +214,7 @@ class PyObjectGlobalVariable_%(module_identifier)s
 
             if ( allow_builtins )
             {
-                entry = GET_PYDICT_ENTRY( _module_builtin, *this->var_name );
+                entry = GET_PYDICT_ENTRY( module_builtin, *this->var_name );
 
                 return entry->me_value != NULL;
             }
@@ -298,6 +272,20 @@ static struct PyModuleDef _moduledef =
   };
 #endif
 
+#define _MODULE_UNFREEZER %(use_unfreezer)d
+
+#if _MODULE_UNFREEZER
+// For embedded modules, to be unpacked. Used by main program/package only
+extern void registerMetaPathBasedUnfreezer( struct _inittab *_frozes_modules );
+
+// Our own inittab for lookup of "frozen" modules, i.e. the ones included in this binary.
+static struct _inittab _frozes_modules[] =
+{
+%(module_inittab)s
+    { NULL, NULL }
+};
+#endif
+
 #ifdef _NUITKA_EXE
 static bool init_done = false;
 #endif
@@ -305,7 +293,7 @@ static bool init_done = false;
 // The exported interface to CPython. On import of the module, this function gets
 // called. It has have that exact function name.
 
-MOD_INIT( %(module_identifier)s )
+MOD_INIT_DECL( %(module_identifier)s )
 {
 #ifdef _NUITKA_EXE
     // Packages can be imported recursively in deep executables.
@@ -330,16 +318,16 @@ MOD_INIT( %(module_identifier)s )
     PyType_Ready( &Nuitka_Generator_Type );
     PyType_Ready( &Nuitka_Function_Type );
     PyType_Ready( &Nuitka_Method_Type );
-    PyType_Ready( &Nuitka_Genexpr_Type );
+
+    patchInspectModule();
+#endif
+
+#if _MODULE_UNFREEZER
+    registerMetaPathBasedUnfreezer( _frozes_modules );
 #endif
 
     // puts( "in init%(module_identifier)s" );
 
-#ifdef _NUITKA_MODULE
-    // Remember it here, Py_InitModule4 will clear it.
-    char const *qualified_name = _Py_PackageContext;
-#endif
-
     // Create the module object first. There are no methods initially, all are added
     // dynamically in actual code only.  Also no __doc__ is initially set, as it could not
     // contain 0 this way, added early in actual code.  No self for modules, we have no
@@ -358,6 +346,18 @@ MOD_INIT( %(module_identifier)s )
 
     assertObject( _module_%(module_identifier)s );
 
+#ifndef _NUITKA_MODULE
+// TODO: Seems to work for Python2.7 as well, and maybe even useful. To be
+// investigated in separate tests.
+#if PYTHON_VERSION >= 300
+    {
+        int r = PyObject_SetItem( PySys_GetObject( (char *)"modules" ), %(module_name_obj)s, _module_%(module_identifier)s );
+
+        assert( r != -1 );
+    }
+#endif
+#endif
+
     // For deep importing of a module we need to have "__builtins__", so we set it
     // ourselves in the same way than CPython does. Note: This must be done before
     // the frame object is allocated, or else it may fail.
@@ -366,7 +366,7 @@ MOD_INIT( %(module_identifier)s )
 
     if ( PyDict_GetItem( module_dict, _python_str_plain___builtins__ ) == NULL )
     {
-        PyObject *value = ( PyObject *)_module_builtin;
+        PyObject *value = ( PyObject *)module_builtin;
 
 #ifndef _NUITKA_MODULE
         if ( _module_%(module_identifier)s != _module___main__ )
@@ -394,7 +394,7 @@ MOD_INIT( %(module_identifier)s )
     }
 #endif
 
-    frame_%(module_identifier)s = MAKE_FRAME( MAKE_CODEOBJ( %(filename_identifier)s, %(module_name_obj)s, 0, _python_tuple_empty, 0 ), _module_%(module_identifier)s);
+    frame_%(module_identifier)s = MAKE_FRAME( %(code_identifier)s, _module_%(module_identifier)s );
 
     // Set module frame as the currently active one.
     FrameGuardLight frame_guard( &frame_%(module_identifier)s );
@@ -445,20 +445,6 @@ module_init_no_package_template = """\
 #if %(is_package)d
     _mvar_%(module_identifier)s___path__.assign0( %(path_identifier)s );
 #endif
-
-#ifdef _NUITKA_MODULE
-    // Set the package attribute from what the import mechanism provided. The package
-    // variable should be set for the module code already.
-    if ( qualified_name )
-    {
-        _mvar_%(module_identifier)s___package__.assign0(
-            PyString_FromStringAndSize(
-               qualified_name,
-               strrchr( qualified_name, '.' ) -  qualified_name
-            )
-        );
-    }
-#endif
 """
 
 module_init_in_package_template = """\
@@ -472,9 +458,9 @@ module_init_in_package_template = """\
     assertObject( _module_%(package_identifier)s );
 
     SET_ATTRIBUTE(
+        _module_%(module_identifier)s,
         _module_%(package_identifier)s,
-        %(module_name)s,
-        _module_%(module_identifier)s
+        %(module_name)s
     );
 """
 
