@@ -3,28 +3,17 @@
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
 #
-#     If you submit patches or make the software available to licensors of
-#     this software in either form, you automatically them grant them a
-#     license for your part of the code under "Apache License 2.0" unless you
-#     choose to remove this notice.
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
 #
-#     Kay Hayen uses the right to license his code under only GPL version 3,
-#     to discourage a fork of Nuitka before it is "finished". He will later
-#     make a new "Nuitka" release fully under "Apache License 2.0".
+#        http://www.apache.org/licenses/LICENSE-2.0
 #
-#     This program is free software: you can redistribute it and/or modify
-#     it under the terms of the GNU General Public License as published by
-#     the Free Software Foundation, version 3 of the License.
-#
-#     This program is distributed in the hope that it will be useful,
-#     but WITHOUT ANY WARRANTY; without even the implied warranty of
-#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#     GNU General Public License for more details.
-#
-#     You should have received a copy of the GNU General Public License
-#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#
-#     Please leave the whole of this copyright notice intact.
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 #
 """ Builtin type nodes tuple/list/float/int etc.
 
@@ -39,11 +28,14 @@ from .NodeBases import (
     CPythonNodeBase
 )
 
-from .NodeMakingHelpers import makeConstantReplacementNode
+from .NodeMakingHelpers import (
+    makeConstantReplacementNode,
+    wrapExpressionWithSideEffects
+)
 
 from nuitka.transform.optimizations import BuiltinOptimization
 
-from nuitka.Utils import getPythonVersion
+from nuitka.Utils import python_version
 
 class CPythonExpressionBuiltinTypeBase( CPythonExpressionBuiltinSingleArgBase ):
     pass
@@ -73,10 +65,6 @@ class CPythonExpressionBuiltinBool( CPythonExpressionBuiltinTypeBase ):
     builtin_spec = BuiltinOptimization.builtin_bool_spec
 
 
-class CPythonExpressionBuiltinStr( CPythonExpressionBuiltinTypeBase ):
-    kind = "EXPRESSION_BUILTIN_STR"
-
-    builtin_spec = BuiltinOptimization.builtin_str_spec
 
 
 class CPythonExpressionBuiltinIntLongBase( CPythonChildrenHaving, CPythonNodeBase, \
@@ -103,7 +91,7 @@ class CPythonExpressionBuiltinIntLongBase( CPythonChildrenHaving, CPythonNodeBas
     getValue = CPythonChildrenHaving.childGetter( "value" )
     getBase = CPythonChildrenHaving.childGetter( "base" )
 
-    def computeNode( self ):
+    def computeNode( self, constraint_collection ):
         value = self.getValue()
         base = self.getBase()
 
@@ -127,17 +115,8 @@ class CPythonExpressionBuiltinInt( CPythonExpressionBuiltinIntLongBase ):
 
     builtin_spec = BuiltinOptimization.builtin_int_spec
 
-
-if getPythonVersion() < 300:
-    class CPythonExpressionBuiltinLong( CPythonExpressionBuiltinIntLongBase ):
-        kind = "EXPRESSION_BUILTIN_LONG"
-
-        builtin_spec = BuiltinOptimization.builtin_long_spec
-
-
-class CPythonExpressionBuiltinUnicode( CPythonChildrenHaving, CPythonNodeBase ):
-    kind = "EXPRESSION_BUILTIN_UNICODE"
-
+class CPythonExpressionBuiltinUnicodeBase( CPythonChildrenHaving, CPythonNodeBase, \
+                                           CPythonExpressionSpecBasedComputationMixin ):
     named_children = ( "value", "encoding", "errors" )
 
     def __init__( self, value, encoding, errors, source_ref ):
@@ -153,3 +132,61 @@ class CPythonExpressionBuiltinUnicode( CPythonChildrenHaving, CPythonNodeBase ):
         )
 
     getValue = CPythonChildrenHaving.childGetter( "value" )
+    getEncoding = CPythonChildrenHaving.childGetter( "encoding" )
+    getErrors = CPythonChildrenHaving.childGetter( "errors" )
+
+    def computeNode( self, constraint_collection ):
+        args = [
+            self.getValue(),
+            self.getEncoding(),
+            self.getErrors()
+        ]
+
+        while args and args[-1] is None:
+            del args[-1]
+
+        return self.computeBuiltinSpec( tuple( args ) )
+
+
+if python_version < 300:
+    class CPythonExpressionBuiltinStr( CPythonExpressionBuiltinTypeBase ):
+        kind = "EXPRESSION_BUILTIN_STR"
+
+        builtin_spec = BuiltinOptimization.builtin_str_spec
+
+        def computeNode( self, constraint_collection ):
+            new_node, change_tags, change_desc = CPythonExpressionBuiltinTypeBase.computeNode(
+                self,
+                constraint_collection
+            )
+
+            if new_node is self:
+                str_value = self.getValue().getStrValue()
+
+                if str_value is not None:
+                    new_node = wrapExpressionWithSideEffects(
+                        new_node = str_value,
+                        old_node = self.getValue()
+                    )
+
+                    change_tags = "new_expression"
+                    change_desc = "Predicted str builtin result"
+
+            return new_node, change_tags, change_desc
+
+
+    class CPythonExpressionBuiltinLong( CPythonExpressionBuiltinIntLongBase ):
+        kind = "EXPRESSION_BUILTIN_LONG"
+
+        builtin_spec = BuiltinOptimization.builtin_long_spec
+
+
+    class CPythonExpressionBuiltinUnicode( CPythonExpressionBuiltinUnicodeBase ):
+        kind = "EXPRESSION_BUILTIN_UNICODE"
+
+        builtin_spec = BuiltinOptimization.builtin_unicode_spec
+else:
+    class CPythonExpressionBuiltinStr( CPythonExpressionBuiltinUnicodeBase ):
+        kind = "EXPRESSION_BUILTIN_STR"
+
+        builtin_spec = BuiltinOptimization.builtin_str_spec
