@@ -3,28 +3,17 @@
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
 #
-#     If you submit patches or make the software available to licensors of
-#     this software in either form, you automatically them grant them a
-#     license for your part of the code under "Apache License 2.0" unless you
-#     choose to remove this notice.
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
 #
-#     Kay Hayen uses the right to license his code under only GPL version 3,
-#     to discourage a fork of Nuitka before it is "finished". He will later
-#     make a new "Nuitka" release fully under "Apache License 2.0".
+#        http://www.apache.org/licenses/LICENSE-2.0
 #
-#     This program is free software: you can redistribute it and/or modify
-#     it under the terms of the GNU General Public License as published by
-#     the Free Software Foundation, version 3 of the License.
-#
-#     This program is distributed in the hope that it will be useful,
-#     but WITHOUT ANY WARRANTY; without even the implied warranty of
-#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#     GNU General Public License for more details.
-#
-#     You should have received a copy of the GNU General Public License
-#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#
-#     Please leave the whole of this copyright notice intact.
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 #
 """ Conditional nodes.
 
@@ -59,7 +48,7 @@ class CPythonExpressionConditional( CPythonExpressionChildrenHavingBase ):
     getExpressionNo = CPythonExpressionChildrenHavingBase.childGetter( "expression_no" )
     getCondition = CPythonExpressionChildrenHavingBase.childGetter( "condition" )
 
-    def computeNode( self ):
+    def computeNode( self, constraint_collection ):
         condition = self.getCondition()
 
         # TODO: Actually really want to check the truth value only, not constant ness.
@@ -101,3 +90,18 @@ class CPythonStatementConditional( CPythonExpressionChildrenHavingBase ):
     setBranchYes = CPythonExpressionChildrenHavingBase.childSetter( "yes_branch" )
     getBranchNo = CPythonExpressionChildrenHavingBase.childGetter( "no_branch" )
     setBranchNo = CPythonExpressionChildrenHavingBase.childSetter( "no_branch" )
+
+    def isStatementAbortative( self ):
+        yes_branch = self.getBranchYes()
+
+        if yes_branch is not None and not yes_branch.isStatementAbortative():
+            return False
+
+        no_branch = self.getBranchNo()
+
+        if no_branch is not None and not no_branch.isStatementAbortative():
+            return False
+
+        assert yes_branch is not None or no_branch is not None
+
+        return yes_branch is not None and no_branch is not None
