@@ -3,28 +3,17 @@
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
 #
-#     If you submit patches or make the software available to licensors of
-#     this software in either form, you automatically them grant them a
-#     license for your part of the code under "Apache License 2.0" unless you
-#     choose to remove this notice.
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
 #
-#     Kay Hayen uses the right to license his code under only GPL version 3,
-#     to discourage a fork of Nuitka before it is "finished". He will later
-#     make a new "Nuitka" release fully under "Apache License 2.0".
+#        http://www.apache.org/licenses/LICENSE-2.0
 #
-#     This program is free software: you can redistribute it and/or modify
-#     it under the terms of the GNU General Public License as published by
-#     the Free Software Foundation, version 3 of the License.
-#
-#     This program is distributed in the hope that it will be useful,
-#     but WITHOUT ANY WARRANTY; without even the implied warranty of
-#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#     GNU General Public License for more details.
-#
-#     You should have received a copy of the GNU General Public License
-#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#
-#     Please leave the whole of this copyright notice intact.
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 #
 """ Nodes related to raising and making exceptions.
 
@@ -32,6 +21,7 @@
 
 from .NodeBases import (
     CPythonExpressionChildrenHavingBase,
+    CPythonExpressionMixin,
     CPythonChildrenHaving,
     CPythonNodeBase
 )
@@ -76,6 +66,8 @@ class CPythonStatementRaiseException( CPythonChildrenHaving, CPythonNodeBase ):
     def markAsReraiseLocal( self ):
         self.reraise_local = True
 
+    def isStatementAbortative( self ):
+        return True
 
 class CPythonExpressionRaiseException( CPythonExpressionChildrenHavingBase ):
     """ This node type is only produced via optimization.
@@ -90,27 +82,21 @@ class CPythonExpressionRaiseException( CPythonExpressionChildrenHavingBase ):
 
     kind = "EXPRESSION_RAISE_EXCEPTION"
 
-    named_children = ( "side_effects", "exception_type", "exception_value" )
+    named_children = ( "exception_type", "exception_value" )
 
-    def __init__( self, exception_type, exception_value, side_effects, source_ref ):
+    def __init__( self, exception_type, exception_value, source_ref ):
         CPythonExpressionChildrenHavingBase.__init__(
             self,
             values     = {
                 "exception_type"  : exception_type,
-                "exception_value" : exception_value,
-                "side_effects"    : tuple( side_effects )
+                "exception_value" : exception_value
             },
             source_ref = source_ref
         )
 
     getExceptionType = CPythonExpressionChildrenHavingBase.childGetter( "exception_type" )
     getExceptionValue = CPythonExpressionChildrenHavingBase.childGetter( "exception_value" )
-    getSideEffects = CPythonExpressionChildrenHavingBase.childGetter( "side_effects" )
-
-    def addSideEffects( self, side_effects ):
-        self.setChild( "side_effects", tuple( side_effects ) + self.getSideEffects() )
-
-    def computeNode( self ):
+    def computeNode( self, constraint_collection ):
         return self, None, None
 
 
@@ -138,5 +124,54 @@ class CPythonExpressionBuiltinMakeException( CPythonExpressionChildrenHavingBase
 
     getArgs = CPythonExpressionChildrenHavingBase.childGetter( "args" )
 
-    def computeNode( self ):
+    def computeNode( self, constraint_collection ):
         return self, None, None
+
+
+class CPythonExpressionCaughtExceptionTypeRef( CPythonNodeBase, CPythonExpressionMixin ):
+    kind = "EXPRESSION_CAUGHT_EXCEPTION_TYPE_REF"
+
+    def __init__( self, source_ref ):
+        CPythonNodeBase.__init__( self, source_ref = source_ref )
+
+    def computeNode( self, constraint_collection ):
+        # TODO: Might be predictable based on the exception handler this is in.
+        return self, None, None
+
+    def mayHaveSideEffects( self, constraint_collection ):
+        # Referencing the expression type has no side effect
+        return False
+
+
+class CPythonExpressionCaughtExceptionValueRef( CPythonNodeBase, CPythonExpressionMixin ):
+    kind = "EXPRESSION_CAUGHT_EXCEPTION_VALUE_REF"
+
+    def __init__( self, source_ref ):
+        CPythonNodeBase.__init__( self, source_ref = source_ref )
+
+    def computeNode( self, constraint_collection ):
+        # TODO: Might be predictable based on the exception handler this is in.
+        return self, None, None
+
+    def mayHaveSideEffects( self, constraint_collection ):
+        # Referencing the expression type has no side effect
+        return False
+
+    def makeCloneAt( self, source_ref ):
+        return CPythonExpressionCaughtExceptionValueRef(
+            source_ref = source_ref
+        )
+
+
+class CPythonExpressionCaughtExceptionTracebackRef( CPythonNodeBase, CPythonExpressionMixin ):
+    kind = "EXPRESSION_CAUGHT_EXCEPTION_TRACEBACK_REF"
+
+    def __init__( self, source_ref ):
+        CPythonNodeBase.__init__( self, source_ref = source_ref )
+
+    def computeNode( self, constraint_collection ):
+        return self, None, None
+
+    def mayHaveSideEffects( self, constraint_collection ):
+        # Referencing the expression type has no side effect
+        return False
