@@ -3,28 +3,17 @@
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
 #
-#     If you submit patches or make the software available to licensors of
-#     this software in either form, you automatically them grant them a
-#     license for your part of the code under "Apache License 2.0" unless you
-#     choose to remove this notice.
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
 #
-#     Kay Hayen uses the right to license his code under only GPL version 3,
-#     to discourage a fork of Nuitka before it is "finished". He will later
-#     make a new "Nuitka" release fully under "Apache License 2.0".
+#        http://www.apache.org/licenses/LICENSE-2.0
 #
-#     This program is free software: you can redistribute it and/or modify
-#     it under the terms of the GNU General Public License as published by
-#     the Free Software Foundation, version 3 of the License.
-#
-#     This program is distributed in the hope that it will be useful,
-#     but WITHOUT ANY WARRANTY; without even the implied warranty of
-#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#     GNU General Public License for more details.
-#
-#     You should have received a copy of the GNU General Public License
-#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#
-#     Please leave the whole of this copyright notice intact.
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 #
 """ Nodes related to importing modules or names.
 
@@ -83,9 +72,15 @@ class CPythonExpressionImportModule( CPythonExpressionChildrenHavingBase ):
         self.attempted_recurse = True
 
     getModule = CPythonExpressionChildrenHavingBase.childGetter( "module" )
-    setModule = CPythonExpressionChildrenHavingBase.childSetter( "module" )
+    _setModule = CPythonExpressionChildrenHavingBase.childSetter( "module" )
 
-    def computeNode( self ):
+    def setModule( self, module ):
+        # Modules have no parent.
+        assert module.parent is None
+        self._setModule( module )
+        module.parent = None
+
+    def computeNode( self, constraint_collection ):
         # TODO: May return a module reference of some sort in the future with embedded
         # modules.
         return self, None, None
@@ -127,7 +122,7 @@ class CPythonExpressionBuiltinImport( CPythonExpressionChildrenHavingBase ):
     getLocals = CPythonExpressionChildrenHavingBase.childGetter( "locals" )
     getLevel = CPythonExpressionChildrenHavingBase.childGetter( "level" )
 
-    def computeNode( self ):
+    def computeNode( self, constraint_collection ):
         module_name = self.getImportName()
         fromlist = self.getFromList()
         level = self.getLevel()
@@ -196,7 +191,7 @@ class CPythonExpressionImportName( CPythonExpressionChildrenHavingBase ):
 
     getModule = CPythonExpressionChildrenHavingBase.childGetter( "module" )
 
-    def computeNode( self ):
+    def computeNode( self, constraint_collection ):
         # TODO: May return a module or module variable reference of some sort in the
         # future with embedded modules.
         return self, None, None
