@@ -3,28 +3,17 @@
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
 #
-#     If you submit patches or make the software available to licensors of
-#     this software in either form, you automatically them grant them a
-#     license for your part of the code under "Apache License 2.0" unless you
-#     choose to remove this notice.
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
 #
-#     Kay Hayen uses the right to license his code under only GPL version 3,
-#     to discourage a fork of Nuitka before it is "finished". He will later
-#     make a new "Nuitka" release fully under "Apache License 2.0".
+#        http://www.apache.org/licenses/LICENSE-2.0
 #
-#     This program is free software: you can redistribute it and/or modify
-#     it under the terms of the GNU General Public License as published by
-#     the Free Software Foundation, version 3 of the License.
-#
-#     This program is distributed in the hope that it will be useful,
-#     but WITHOUT ANY WARRANTY; without even the implied warranty of
-#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#     GNU General Public License for more details.
-#
-#     You should have received a copy of the GNU General Public License
-#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#
-#     Please leave the whole of this copyright notice intact.
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 #
 """ Node base classes.
 
@@ -36,13 +25,14 @@ These classes provide the generic base classes available for nodes.
 from nuitka.odict import OrderedDict
 
 from nuitka import (
-    Variables,
     Tracing,
     TreeXML
 )
 
 from . import UsageCheck
 
+from nuitka.__past__ import iterItems
+
 lxml = TreeXML.lxml
 
 class NodeCheckMetaClass( type ):
@@ -137,7 +127,7 @@ class CPythonNodeBase( CPythonNodeMetaClassBase ):
         except:
             detail = "detail raises exception"
 
-        if detail == "":
+        if not detail:
             return "<Node %s>" % self.getDescription()
         else:
             return "<Node %s %s>" % ( self.getDescription(), detail )
@@ -160,7 +150,7 @@ class CPythonNodeBase( CPythonNodeMetaClassBase ):
 
         """
         # Virtual method, pylint: disable=R0201
-        return ""
+        return self.getDetails()
 
     def getParent( self ):
         """ Parent of the node. Every node except modules have to have a parent.
@@ -196,18 +186,6 @@ class CPythonNodeBase( CPythonNodeMetaClassBase ):
 
         return parent
 
-    def getParentClass( self ):
-        """ Return the parent that is a class body.
-
-        """
-
-        parent = self.getParent()
-
-        while parent is not None and not parent.isExpressionClassBody():
-            parent = parent.getParent()
-
-        return parent
-
     def getParentModule( self ):
         """ Return the parent that is module.
 
@@ -224,9 +202,9 @@ class CPythonNodeBase( CPythonNodeMetaClassBase ):
         return parent
 
     def isParentVariableProvider( self ):
-        # Check if it's a closure giver or a class, in which cases it can provide
-        # variables, pylint: disable=E1101
-        return isinstance( self, CPythonClosureGiverNodeBase ) or self.isExpressionClassBody()
+        # Check if it's a closure giver, in which cases it can provide variables,
+        # pylint: disable=E1101
+        return isinstance( self, CPythonClosureGiverNodeBase )
 
     def isClosureVariableTaker( self ):
         return self.hasTag( "closure_taker" )
@@ -249,7 +227,7 @@ class CPythonNodeBase( CPythonNodeMetaClassBase ):
             line = "%s" % self.getSourceReference().getLineNumber()
         )
 
-        for key, value in self.getDetails().iteritems():
+        for key, value in iterItems( self.getDetails() ):
             value = str( value )
 
             if value.startswith( "<" ) and value.endswith( ">" ):
@@ -303,9 +281,6 @@ class CPythonNodeBase( CPythonNodeMetaClassBase ):
     def isExpressionOperationBool2( self ):
         return self.kind.startswith( "EXPRESSION_BOOL_" )
 
-    def isAssignTargetSomething( self ):
-        return self.kind.startswith( "ASSIGN_" )
-
     def isExpressionMakeSequence( self ):
         # Virtual method, pylint: disable=R0201,W0613
         return False
@@ -329,8 +304,7 @@ class CPythonNodeBase( CPythonNodeMetaClassBase ):
         return ()
 
     def getVisitableNodesNamed( self ):
-        assert self.getVisitableNodes.im_class == self.getVisitableNodesNamed.im_class, self.getVisitableNodes.im_class
-
+        # Virtual method, pylint: disable=R0201
         return ()
 
     def getChildNodesNotTagged( self, tag ):
@@ -356,12 +330,18 @@ class CPythonNodeBase( CPythonNodeMetaClassBase ):
         # Virtual method, pylint: disable=R0201,W0613
         return None
 
-    def mayHaveSideEffects( self ):
+    def mayHaveSideEffects( self, constraint_collection ):
         """ Unless we are told otherwise, everything may have a side effect. """
         # Virtual method, pylint: disable=R0201,W0613
 
         return True
 
+    def extractSideEffects( self ):
+        """ Unless defined otherwise, the expression is the side effect. """
+        # Virtual method, pylint: disable=R0201,W0613
+
+        return ( self, )
+
     def mayRaiseException( self, exception_type ):
         """ Unless we are told otherwise, everything may raise everything. """
         # Virtual method, pylint: disable=R0201,W0613
@@ -374,14 +354,32 @@ class CPythonNodeBase( CPythonNodeMetaClassBase ):
 
         return False
 
+    def isStatementAbortative( self ):
+        """ Is the node abortative, control flow doesn't continue after this node.  """
+        # Virtual method, pylint: disable=R0201
+        assert self.isStatement(), self.kind
+
+        return False
+
+    def needsLocalsDict( self ):
+        """ Node requires a locals dictionary by provider. """
+
+        # Virtual method, pylint: disable=R0201,W0613
+        return False
+
     def hasTag( self, tag ):
         return tag in self.__class__.tags
 
+    def getIntegerValue( self, constraint_collection ):
+        """ Node as integer value, if possible."""
+        # Virtual method, pylint: disable=R0201,W0613
+
+
 class CPythonCodeNodeBase( CPythonNodeBase ):
     def __init__( self, name, code_prefix, source_ref ):
         assert name is not None
-        assert " " not in name
-        assert "<" not in name
+        assert " " not in name, name
+        assert "<" not in name, name
 
         CPythonNodeBase.__init__( self, source_ref = source_ref )
 
@@ -431,7 +429,7 @@ class CPythonCodeNodeBase( CPythonNodeBase ):
 
             uid = "_%d" % search.getChildUID( self )
 
-            if isinstance( self, CPythonCodeNodeBase ):
+            if isinstance( self, CPythonCodeNodeBase ) and self.name:
                 name = uid + "_" + self.name
             else:
                 name = uid
@@ -553,15 +551,14 @@ class CPythonChildrenHaving:
                 pass
             elif type( value ) is tuple:
                 if old_node in value:
-                    new_value = []
-
-                    for val in value:
-                        if val is not old_node:
-                            new_value.append( val )
-                        else:
-                            new_value.append( new_node )
-
-                    self.setChild( key, tuple( new_value ) )
+                    self.setChild(
+                        key,
+                        tuple(
+                            (val if val is not old_node else new_node)
+                            for val in
+                            value
+                        )
+                    )
 
                     break
             elif isinstance( value, CPythonNodeBase ):
@@ -582,6 +579,39 @@ class CPythonChildrenHaving:
         if new_node is not None:
             new_node.parent = old_node.parent
 
+    def makeCloneAt( self, source_ref ):
+        values = {}
+
+        for key, value in self.child_values.items():
+            assert type( value ) is not list, key
+
+            if value is None:
+                values[ key ] = None
+            elif type( value ) is tuple:
+                values[ key ] = tuple(
+                    v.makeCloneAt(
+                        source_ref = v.getSourceReference()
+                    )
+                    for v in
+                    value
+                )
+            else:
+                values[ key ] = value.makeCloneAt(
+                    value.getSourceReference()
+                )
+
+        values.update( self.getDetails() )
+
+        try:
+            return self.__class__(
+                source_ref = source_ref,
+                **values
+            )
+        except TypeError as e:
+            print( "Problem cloning", self.__class__ )
+
+            raise
+
 
 class CPythonClosureGiverNodeBase( CPythonCodeNodeBase ):
     """ Mixin for nodes that provide variables for closure takers. """
@@ -643,6 +673,7 @@ class CPythonClosureGiverNodeBase( CPythonCodeNodeBase ):
                 if not usages:
                     del self.providing[ variable.getName() ]
 
+
 class CPythonParameterHavingNodeBase( CPythonClosureGiverNodeBase ):
     def __init__( self, name, code_prefix, parameters, source_ref ):
         CPythonClosureGiverNodeBase.__init__(
@@ -668,17 +699,17 @@ class CPythonClosureTaker:
 
     tags = ( "closure_taker", )
 
-    def __init__( self, provider ):
-        assert self.__class__.early_closure is not None, self
-
+    def __init__( self, provider, early_closure ):
         assert provider.isParentVariableProvider(), provider
 
         self.provider = provider
+        self.early_closure = early_closure
 
         self.taken = set()
 
         self.temp_variables = set()
 
+        self.temp_keeper_count = 0
 
     def getParentVariableProvider( self ):
         return self.provider
@@ -732,37 +763,27 @@ class CPythonClosureTaker:
         else:
             return None
 
-    # Normally it's good to lookup name references immediately, but in case of a function
-    # body it is not allowed to do that, because a later assignment needs to be queried
-    # first. Nodes need to indicate via this if they would like to resolve references at
-    # the same time as assignments.
-    early_closure = None
-
     def isEarlyClosure( self ):
+        """ Normally it's good to lookup name references immediately, but not for functions.
+
+        in case of a function body it is not allowed to do that, because a later
+        assignment needs to be queried first. Nodes need to indicate via this if they
+        would like to resolve references at the same time as assignments.
+        """
+
         return self.early_closure
 
-    def getTempVariable( self ):
-        result = Variables.TempVariable(
-            owner         = self,
-            variable_name = "__tmp_%d" % len( self.temp_variables )
-        )
-        self.temp_variables.add( result )
-        return result
+    def allocateTempKeeperName( self ):
+        self.temp_keeper_count += 1
 
-    def getTempVariables( self ):
-        return tuple( self.temp_variables )
-
-    def setTempVariables( self, variables ):
-        for variable in variables:
-            assert variable.isTempVariable(), variable
-            assert variable.getOwner() is self
-
-        self.temp_variables = variables
+        return "keeper_%d" % self.temp_keeper_count
 
+    def getTempKeeperNames( self ):
+        return tuple( "keeper_%d" % ( i+1 ) for i in range( self.temp_keeper_count ) )
 
 
 class CPythonExpressionMixin:
-    def getValueFriend( self ):
+    def getValueFriend( self, constraint_collection ):
         return self
 
     def isCompileTimeConstant( self ):
@@ -776,9 +797,18 @@ class CPythonExpressionMixin:
         return False
 
     def getCompileTimeConstant( self ):
-        assert self.isCompileTimeConstant()
+        assert self.isCompileTimeConstant(), ( self, "asked for compile time constant" )
+
         assert False
 
+    def getTruthValue( self, constraint_collection ):
+        """ Return known truth value. The "None" value indicates unknown. """
+
+        if self.isCompileTimeConstant():
+            return bool( self.getCompileTimeConstant() )
+        else:
+            return None
+
     def isKnownToBeIterable( self, count ):
         """ Can be iterated at all (count is None) or exactly count times.
 
@@ -789,6 +819,49 @@ class CPythonExpressionMixin:
         # Virtual method, pylint: disable=R0201,W0613
         return False
 
+    def isKnownToBeIterableAtMin( self, count, constraint_collection ):
+        # Virtual method, pylint: disable=R0201,W0613
+        return False
+
+    def isKnownToBeIterableAtMax( self, count, constraint_collection ):
+        # Virtual method, pylint: disable=R0201,W0613
+        return False
+
+    def getIterationNext( self, constraint_collection ):
+        # Virtual method, pylint: disable=R0201,W0613
+        return None
+
+    def mayProvideReference( self ):
+        """ May at run time produce a reference.
+
+        This then would have to be consumed or released in a reliable way.
+        """
+
+        # Virtual method, pylint: disable=R0201
+        return True
+
+    def getIterationLength( self, constraint_collection ):
+        """ Value that "len" or "PyObject_Size" would give, if known.
+
+        Otherwise it is "None" to indicate unknown.
+        """
+
+        # Virtual method, pylint: disable=R0201
+        return None
+
+    def getStrValue( self ):
+        """ Value that "str" or "PyObject_Str" would give, if known.
+
+        Otherwise it is "None" to indicate unknown.
+        """
+
+        return None
+
+    def onRelease( self, constraint_collection ):
+        # print "onRelease", self
+        pass
+
+
 
 class CPythonExpressionSpecBasedComputationMixin( CPythonExpressionMixin ):
     builtin_spec = None
@@ -797,9 +870,12 @@ class CPythonExpressionSpecBasedComputationMixin( CPythonExpressionMixin ):
         assert self.builtin_spec is not None, self
 
         for value in given_values:
-            if not value.isCompileTimeConstant():
+            if value is not None and not value.isCompileTimeConstant():
                 return self, None, None
 
+        if not self.builtin_spec.isCompileTimeComputable( given_values ):
+            return self, None, None
+
         from .NodeMakingHelpers import getComputationResult
 
         return getComputationResult(
@@ -818,6 +894,26 @@ class CPythonExpressionChildrenHavingBase( CPythonChildrenHaving, CPythonNodeBas
             values = values
         )
 
+
+class CPythonExpressionBuiltinNoArgBase( CPythonNodeBase, CPythonExpressionMixin ):
+    def __init__( self, builtin_function, source_ref ):
+        CPythonNodeBase.__init__(
+            self,
+            source_ref = source_ref
+        )
+
+        self.builtin_function = builtin_function
+
+    def computeNode( self, constraint_collection ):
+        from .NodeMakingHelpers import getComputationResult
+
+        return getComputationResult(
+            node        = self,
+            computation = lambda : self.builtin_function(),
+            description = "No arg %s builtin" % self.builtin_function.__name__
+        )
+
+
 class CPythonExpressionBuiltinSingleArgBase( CPythonExpressionChildrenHavingBase, \
                                              CPythonExpressionSpecBasedComputationMixin ):
     named_children = ( "value", )
@@ -833,7 +929,7 @@ class CPythonExpressionBuiltinSingleArgBase( CPythonExpressionChildrenHavingBase
 
     getValue = CPythonExpressionChildrenHavingBase.childGetter( "value" )
 
-    def computeNode( self ):
+    def computeNode( self, constraint_collection ):
         value = self.getValue()
 
         assert self.builtin_spec is not None, self
@@ -842,3 +938,22 @@ class CPythonExpressionBuiltinSingleArgBase( CPythonExpressionChildrenHavingBase
             return self.computeBuiltinSpec( () )
         else:
             return self.computeBuiltinSpec( ( value, ) )
+
+
+class CPythonSideEffectsFromChildrenMixin:
+    def mayHaveSideEffects( self, constraint_collection ):
+        for child in self.getVisitableNodes():
+            if child.mayHaveSideEffects( constraint_collection ):
+                return True
+        else:
+            return False
+
+    def extractSideEffects( self ):
+        # No side effects at all but from the children.
+
+        result = []
+
+        for child in self.getVisitableNodes():
+            result.extend( child.extractSideEffects() )
+
+        return tuple( result )
