@@ -3,28 +3,17 @@
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
 #
-#     If you submit patches or make the software available to licensors of
-#     this software in either form, you automatically them grant them a
-#     license for your part of the code under "Apache License 2.0" unless you
-#     choose to remove this notice.
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
 #
-#     Kay Hayen uses the right to license his code under only GPL version 3,
-#     to discourage a fork of Nuitka before it is "finished". He will later
-#     make a new "Nuitka" release fully under "Apache License 2.0".
+#        http://www.apache.org/licenses/LICENSE-2.0
 #
-#     This program is free software: you can redistribute it and/or modify
-#     it under the terms of the GNU General Public License as published by
-#     the Free Software Foundation, version 3 of the License.
-#
-#     This program is distributed in the hope that it will be useful,
-#     but WITHOUT ANY WARRANTY; without even the implied warranty of
-#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#     GNU General Public License for more details.
-#
-#     You should have received a copy of the GNU General Public License
-#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#
-#     Please leave the whole of this copyright notice intact.
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 #
 """ These are just helpers to create nodes, often to replace existing nodes
 
@@ -49,6 +38,8 @@ from .StatementNodes import (
     CPythonStatementsSequence
 )
 
+from .SideEffectNode import CPythonExpressionSideEffects
+
 def makeConstantReplacementNode( constant, node ):
     return CPythonExpressionConstantRef(
         constant   = constant,
@@ -81,7 +72,6 @@ def makeRaiseExceptionReplacementExpression( expression, exception_type, excepti
             constant = exception_value,
             node     = expression
         ),
-        side_effects    = (),
         source_ref      = source_ref
     )
 
@@ -147,34 +137,6 @@ def makeRaiseExceptionReplacementStatement( statement, exception_type, exception
 
     return result
 
-def convertRaiseExceptionExpressionRaiseExceptionStatement( node ):
-    assert node.isExpressionRaiseException()
-
-    side_effects = node.getSideEffects()
-
-    raise_node = CPythonStatementRaiseException(
-        exception_type  = node.getExceptionType(),
-        exception_value = node.getExceptionValue(),
-        exception_trace = None,
-        source_ref      = node.getSourceReference()
-    )
-
-    if side_effects:
-        side_effects = tuple(
-            CPythonStatementExpressionOnly(
-                expression = side_effect,
-                source_ref = side_effect.getSourceReference()
-            )
-            for side_effect in side_effects
-        )
-
-        return makeStatementsSequenceReplacementNode(
-            statements = side_effects + ( raise_node, ),
-            node       = node
-        )
-    else:
-        return raise_node
-
 def makeStatementExpressionOnlyReplacementNode( expression, node ):
     return CPythonStatementExpressionOnly(
         expression = expression,
@@ -194,3 +156,44 @@ def convertNoneConstantToNone( node ):
         return None
     else:
         return node
+
+def wrapExpressionWithSideEffects( new_node, old_node ):
+    assert new_node.isExpression()
+
+    side_effects = old_node.extractSideEffects()
+
+    if side_effects:
+        new_node = CPythonExpressionSideEffects(
+            expression   = new_node,
+            side_effects = side_effects,
+            source_ref   = old_node.getSourceReference()
+        )
+
+    return new_node
+
+def wrapStatementWithSideEffects( new_node, old_node, allow_none = False ):
+    assert new_node is not None or allow_none
+
+    side_effects = old_node.extractSideEffects()
+
+    if side_effects:
+        side_effects = tuple(
+            CPythonStatementExpressionOnly(
+                expression = side_effect,
+                source_ref = side_effect.getSourceReference()
+            )
+            for side_effect in side_effects
+        )
+
+        if new_node is not None:
+            new_node = makeStatementsSequenceReplacementNode(
+                statements = side_effects + ( new_node, ),
+                node       = old_node
+            )
+        else:
+            new_node = makeStatementsSequenceReplacementNode(
+                statements = side_effects,
+                node       = old_node
+            )
+
+    return new_node
