@@ -3,28 +3,17 @@
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
 #
-#     If you submit patches or make the software available to licensors of
-#     this software in either form, you automatically them grant them a
-#     license for your part of the code under "Apache License 2.0" unless you
-#     choose to remove this notice.
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
 #
-#     Kay Hayen uses the right to license his code under only GPL version 3,
-#     to discourage a fork of Nuitka before it is "finished". He will later
-#     make a new "Nuitka" release fully under "Apache License 2.0".
+#        http://www.apache.org/licenses/LICENSE-2.0
 #
-#     This program is free software: you can redistribute it and/or modify
-#     it under the terms of the GNU General Public License as published by
-#     the Free Software Foundation, version 3 of the License.
-#
-#     This program is distributed in the hope that it will be useful,
-#     but WITHOUT ANY WARRANTY; without even the implied warranty of
-#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#     GNU General Public License for more details.
-#
-#     You should have received a copy of the GNU General Public License
-#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#
-#     Please leave the whole of this copyright notice intact.
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 #
 """ Nodes for statements.
 
@@ -37,7 +26,7 @@ def mergeStatements( statements ):
     merged_statements = []
 
     for statement in statements:
-        if statement.isStatement():
+        if statement.isStatement() or statement.isStatementsFrame():
             merged_statements.append( statement )
         elif statement.isStatementsSequence():
             merged_statements.extend( mergeStatements( statement.getStatements() ) )
@@ -46,6 +35,7 @@ def mergeStatements( statements ):
 
     return merged_statements
 
+
 class CPythonStatementsSequence( CPythonChildrenHaving, CPythonNodeBase ):
     kind = "STATEMENTS_SEQUENCE"
 
@@ -112,14 +102,45 @@ class CPythonStatementsSequence( CPythonChildrenHaving, CPythonNodeBase ):
         self.setChild( "statements", new_statements )
 
 
-    def mayHaveSideEffects( self ):
+    def mayHaveSideEffects( self, constraint_collection ):
+        assert constraint_collection is None
+
         # Statement sequences have a side effect if one of the statements does.
         for statement in self.getStatements():
-            if statement.mayHaveSideEffects():
+            if statement.mayHaveSideEffects( None ):
                 return True
         else:
             return False
 
+    def isStatementAbortative( self ):
+        return self.getStatements()[-1].isStatementAbortative()
+
+
+class CPythonStatementsFrame( CPythonStatementsSequence ):
+    kind = "STATEMENTS_FRAME"
+
+    def __init__( self, statements, code_name, arg_names, source_ref ):
+        CPythonStatementsSequence.__init__(
+            self,
+            statements = statements,
+            source_ref = source_ref
+        )
+
+        self.arg_names = tuple( arg_names )
+        self.code_name = code_name
+
+    def getDetails( self ):
+        return {
+            "code_name" : self.code_name,
+            "arg_names" : self.arg_names
+        }
+
+    def getArgNames( self ):
+        return self.arg_names
+
+    def getCodeObjectName( self ):
+        return self.code_name
+
 
 class CPythonStatementExpressionOnly( CPythonChildrenHaving, CPythonNodeBase ):
     kind = "STATEMENT_EXPRESSION_ONLY"
@@ -139,19 +160,7 @@ class CPythonStatementExpressionOnly( CPythonChildrenHaving, CPythonNodeBase ):
     def getDetail( self ):
         return "expression %s" % self.getExpression()
 
-    def mayHaveSideEffects( self ):
-        return self.getExpression().mayHaveSideEffects()
+    def mayHaveSideEffects( self, constraint_collection ):
+        return self.getExpression().mayHaveSideEffects( constraint_collection )
 
     getExpression = CPythonChildrenHaving.childGetter( "expression" )
-
-
-class CPythonStatementDeclareGlobal( CPythonNodeBase ):
-    kind = "STATEMENT_DECLARE_GLOBAL"
-
-    def __init__( self, variable_names, source_ref ):
-        self.variable_names = tuple( variable_names )
-
-        CPythonNodeBase.__init__( self, source_ref = source_ref )
-
-    def getVariableNames( self ):
-        return self.variable_names
