@@ -3,28 +3,17 @@
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
 #
-#     If you submit patches or make the software available to licensors of
-#     this software in either form, you automatically them grant them a
-#     license for your part of the code under "Apache License 2.0" unless you
-#     choose to remove this notice.
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
 #
-#     Kay Hayen uses the right to license his code under only GPL version 3,
-#     to discourage a fork of Nuitka before it is "finished". He will later
-#     make a new "Nuitka" release fully under "Apache License 2.0".
+#        http://www.apache.org/licenses/LICENSE-2.0
 #
-#     This program is free software: you can redistribute it and/or modify
-#     it under the terms of the GNU General Public License as published by
-#     the Free Software Foundation, version 3 of the License.
-#
-#     This program is distributed in the hope that it will be useful,
-#     but WITHOUT ANY WARRANTY; without even the implied warranty of
-#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#     GNU General Public License for more details.
-#
-#     You should have received a copy of the GNU General Public License
-#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#
-#     Please leave the whole of this copyright notice intact.
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 #
 """ Node for variable references.
 
@@ -33,7 +22,7 @@ expressions, changing the meaning of course dramatically.
 
 """
 
-from nuitka import Variables, Builtins
+from nuitka import Variables, Builtins, Options
 
 from .NodeBases import CPythonChildrenHaving, CPythonNodeBase, CPythonExpressionMixin
 
@@ -48,6 +37,7 @@ def _isReadOnlyModuleVariable( variable ):
     return ( variable.isModuleVariable() and variable.getReadOnlyIndicator() is True ) or \
            variable.isMaybeLocalVariable()
 
+
 class CPythonExpressionVariableRef( CPythonNodeBase, CPythonExpressionMixin ):
     kind = "EXPRESSION_VARIABLE_REF"
 
@@ -70,13 +60,15 @@ class CPythonExpressionVariableRef( CPythonNodeBase, CPythonExpressionMixin ):
             return repr( self.variable )
 
     def makeCloneAt( self, source_ref ):
-        assert self.variable is None
-
-        return CPythonExpressionVariableRef(
+        result = self.__class__(
             variable_name = self.variable_name,
             source_ref    = source_ref
         )
 
+        result.variable = self.variable
+
+        return result
+
     def getVariableName( self ):
         return self.variable_name
 
@@ -90,7 +82,7 @@ class CPythonExpressionVariableRef( CPythonNodeBase, CPythonExpressionMixin ):
 
         self.variable = variable
 
-    def computeNode( self ):
+    def computeNode( self, constraint_collection ):
         assert self.variable is not None
 
         if _isReadOnlyModuleVariable( self.variable ):
@@ -153,11 +145,48 @@ class CPythonExpressionVariableRef( CPythonNodeBase, CPythonExpressionMixin ):
 
             return new_node, change_tags, change_desc
 
+        # TODO: Enable the below, once we can trust that the corruption of mutable
+        # constants is detected.
+        if not Options.isExperimental():
+            return self, None, None
+
+        friend = constraint_collection.getVariableValueFriend( self.variable )
+
+        if friend is not None and not friend.mayHaveSideEffects( None ) and friend.isNode():
+            assert hasattr( friend, "makeCloneAt" ), friend
+
+            new_node = friend.makeCloneAt(
+                source_ref = self.source_ref,
+            )
+
+            change_desc = "Assignment source of '%s' propagated, as it has no side effects." % self.variable.getName()
+
+            return new_node, "new_expression", change_desc
+
         return self, None, None
 
     def isKnownToBeIterable( self, count ):
         return None
 
+    def mayHaveSideEffects( self, constraint_collection ):
+        if constraint_collection is None:
+            return True
+
+        friend = constraint_collection.getVariableValueFriend( self.variable )
+
+        if friend is not None:
+            # TODO: There is no friend that say "known to not be defined"
+            return False
+        else:
+            return True
+
+
+class CPythonExpressionTargetVariableRef( CPythonExpressionVariableRef ):
+    kind = "EXPRESSION_TARGET_VARIABLE_REF"
+
+    def computeNode( self, constraint_collection ):
+        assert False
+
 
 class CPythonExpressionTempVariableRef( CPythonNodeBase, CPythonExpressionMixin ):
     kind = "EXPRESSION_TEMP_VARIABLE_REF"
@@ -179,10 +208,46 @@ class CPythonExpressionTempVariableRef( CPythonNodeBase, CPythonExpressionMixin
     def getVariable( self ):
         return self.variable
 
-    def computeNode( self ):
+    def computeNode( self, constraint_collection ):
         # Nothing to do here.
         return self, None, None
 
+    def mayRaiseException( self, exception_type ):
+        # Can't happen
+        return False
+
+    def isKnownToBeIterableAtMin( self, count, constraint_collection ):
+        friend = constraint_collection.getVariableValueFriend( self.variable )
+
+        if friend is not None:
+            return friend.isKnownToBeIterableAtMin(
+                count                 = count,
+                constraint_collection = constraint_collection
+            )
+        else:
+            return None
+
+    def isKnownToBeIterableAtMax( self, count, constraint_collection ):
+        friend = constraint_collection.getVariableValueFriend( self.variable )
+
+        if friend is not None:
+            return friend.isKnownToBeIterableAtMax(
+                count                 = count,
+                constraint_collection = constraint_collection
+            )
+        else:
+            return None
+
+    def getIterationNext( self, constraint_collection ):
+        friend = constraint_collection.getVariableValueFriend( self.variable )
+
+        if friend is not None:
+            return friend.getIterationNext(
+                constraint_collection = constraint_collection
+            )
+        else:
+            return None
+
 
 class CPythonStatementTempBlock( CPythonChildrenHaving, CPythonNodeBase ):
     kind = "STATEMENT_TEMP_BLOCK"
@@ -193,7 +258,10 @@ class CPythonStatementTempBlock( CPythonChildrenHaving, CPythonNodeBase ):
     setBody = CPythonChildrenHaving.childSetter( "body" )
 
     def __init__( self, source_ref ):
-        CPythonNodeBase.__init__( self, source_ref = source_ref )
+        CPythonNodeBase.__init__(
+            self,
+            source_ref = source_ref.atInternal()
+        )
 
         CPythonChildrenHaving.__init__(
             self,
@@ -209,7 +277,7 @@ class CPythonStatementTempBlock( CPythonChildrenHaving, CPythonNodeBase ):
 
         result = Variables.TempVariable(
             owner         = self,
-            variable_name = "__tmp_%s" % name
+            variable_name = name
         )
 
         self.temp_variables.add( result )
@@ -218,3 +286,6 @@ class CPythonStatementTempBlock( CPythonChildrenHaving, CPythonNodeBase ):
 
     def getTempVariables( self ):
         return self.temp_variables
+
+    def mayHaveSideEffects( self, constraint_collection ):
+        return self.getBody().mayHaveSideEffects( constraint_collection )
