@@ -3,28 +3,17 @@
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
 #
-#     If you submit patches or make the software available to licensors of
-#     this software in either form, you automatically them grant them a
-#     license for your part of the code under "Apache License 2.0" unless you
-#     choose to remove this notice.
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
 #
-#     Kay Hayen uses the right to license his code under only GPL version 3,
-#     to discourage a fork of Nuitka before it is "finished". He will later
-#     make a new "Nuitka" release fully under "Apache License 2.0".
+#        http://www.apache.org/licenses/LICENSE-2.0
 #
-#     This program is free software: you can redistribute it and/or modify
-#     it under the terms of the GNU General Public License as published by
-#     the Free Software Foundation, version 3 of the License.
-#
-#     This program is distributed in the hope that it will be useful,
-#     but WITHOUT ANY WARRANTY; without even the implied warranty of
-#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#     GNU General Public License for more details.
-#
-#     You should have received a copy of the GNU General Public License
-#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#
-#     Please leave the whole of this copyright notice intact.
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 #
 """ Optimize calls to builtins reference builtin nodes.
 
@@ -34,6 +23,8 @@ supported builtin types.
 
 from .registry import CallRegistry
 
+from nuitka.Utils import python_version
+
 from nuitka.nodes.BuiltinIteratorNodes import (
     CPythonExpressionBuiltinNext1,
     CPythonExpressionBuiltinNext2,
@@ -42,7 +33,6 @@ from nuitka.nodes.BuiltinIteratorNodes import (
     CPythonExpressionBuiltinLen
 )
 from nuitka.nodes.BuiltinTypeNodes import (
-#    CPythonExpressionBuiltinUnicode, TODO: Missing from here actually
     CPythonExpressionBuiltinFloat,
     CPythonExpressionBuiltinTuple,
     CPythonExpressionBuiltinList,
@@ -57,18 +47,27 @@ from nuitka.nodes.BuiltinFormatNodes import (
 )
 from nuitka.nodes.BuiltinDecodingNodes import (
     CPythonExpressionBuiltinChr,
-    CPythonExpressionBuiltinOrd
+    CPythonExpressionBuiltinOrd,
+    CPythonExpressionBuiltinOrd0
 )
 from nuitka.nodes.ExecEvalNodes import (
     CPythonExpressionBuiltinEval,
-    CPythonExpressionBuiltinExec,
-    CPythonExpressionBuiltinExecfile,
+
     CPythonStatementExec
 )
+
+if python_version < 300:
+    from nuitka.nodes.ExecEvalNodes import CPythonExpressionBuiltinExecfile
+else:
+    from nuitka.nodes.ExecEvalNodes import CPythonExpressionBuiltinExec
+
+from nuitka.nodes.VariableRefNode import CPythonExpressionVariableRef
+
 from nuitka.nodes.GlobalsLocalsNodes import (
     CPythonExpressionBuiltinGlobals,
     CPythonExpressionBuiltinLocals,
-    CPythonExpressionBuiltinDir0
+    CPythonExpressionBuiltinDir0,
+    CPythonExpressionBuiltinDir1
 )
 from nuitka.nodes.BuiltinReferenceNodes import (
     CPythonExpressionBuiltinExceptionRef,
@@ -78,36 +77,46 @@ from nuitka.nodes.OperatorNodes import CPythonExpressionOperationUnary
 from nuitka.nodes.ConstantRefNode import CPythonExpressionConstantRef
 from nuitka.nodes.BuiltinDictNode import CPythonExpressionBuiltinDict
 from nuitka.nodes.BuiltinOpenNode import CPythonExpressionBuiltinOpen
-from nuitka.nodes.BuiltinRangeNode import CPythonExpressionBuiltinRange
+from nuitka.nodes.BuiltinRangeNode import (
+    CPythonExpressionBuiltinRange0,
+    CPythonExpressionBuiltinRange1,
+    CPythonExpressionBuiltinRange2,
+    CPythonExpressionBuiltinRange3
+)
+
 from nuitka.nodes.BuiltinVarsNode import CPythonExpressionBuiltinVars
 from nuitka.nodes.ImportNodes import CPythonExpressionBuiltinImport
-from nuitka.nodes.TypeNode import CPythonExpressionBuiltinType1
+from nuitka.nodes.TypeNode import (
+    CPythonExpressionBuiltinSuper,
+    CPythonExpressionBuiltinType1,
+)
 from nuitka.nodes.ClassNodes import CPythonExpressionBuiltinType3
-from nuitka.nodes.CallNode import CPythonExpressionFunctionCall
+from nuitka.nodes.CallNode import CPythonExpressionCall
 from nuitka.nodes.AttributeNode import CPythonExpressionAttributeLookup
 
 from nuitka.nodes.NodeMakingHelpers import (
     makeRaiseExceptionReplacementExpressionFromInstance,
+    makeRaiseExceptionReplacementExpression
 )
 
 
 from nuitka.transform.optimizations import BuiltinOptimization
 
-from nuitka.Utils import getPythonVersion
-
 def dir_extractor( node ):
     # Only treat the empty dir() call, leave the others alone for now.
-    if not node.isEmptyCall():
-        return None
-
-    return CPythonExpressionBuiltinDir0(
-        source_ref = node.getSourceReference()
-    )
+    if node.isEmptyCall():
+        return CPythonExpressionBuiltinDir0(
+            source_ref = node.getSourceReference()
+        )
+    else:
+        return BuiltinOptimization.extractBuiltinArgs(
+            node          = node,
+            builtin_class = CPythonExpressionBuiltinDir1,
+            builtin_spec  = BuiltinOptimization.builtin_dir_spec
+        )
 
 def vars_extractor( node ):
-    positional_args = node.getPositionalArguments()
-
-    if len( positional_args ) == 0:
+    if node.isEmptyCall():
         if node.getParentVariableProvider().isModule():
             return CPythonExpressionBuiltinGlobals(
                 source_ref = node.getSourceReference()
@@ -116,13 +125,12 @@ def vars_extractor( node ):
             return CPythonExpressionBuiltinLocals(
                 source_ref = node.getSourceReference()
             )
-    elif len( positional_args ) == 1:
-        return CPythonExpressionBuiltinVars(
-            source     = positional_args[ 0 ],
-            source_ref = node.getSourceReference()
-        )
     else:
-        assert False
+        return BuiltinOptimization.extractBuiltinArgs(
+            node          = node,
+            builtin_class = CPythonExpressionBuiltinVars,
+            builtin_spec  = BuiltinOptimization.builtin_vars_spec
+        )
 
 def import_extractor( node ):
     return BuiltinOptimization.extractBuiltinArgs(
@@ -181,25 +189,25 @@ def dict_extractor( node ):
     # The dict is a bit strange in that it accepts a position parameter, or not, but
     # won't have a default.
 
-    def wrapExpressionBuiltinDictCreation( positional_args, pairs, source_ref ):
+    def wrapExpressionBuiltinDictCreation( positional_args, dict_star_arg, source_ref ):
         if len( positional_args ) > 1:
-            return CPythonExpressionFunctionCall(
-                called_expression = makeRaiseExceptionReplacementExpressionFromInstance(
+            return CPythonExpressionCall(
+                called          = makeRaiseExceptionReplacementExpressionFromInstance(
                     expression     = node,
                     exception      = TypeError(
                         "dict expected at most 1 arguments, got %d" % len( positional_args )
                     )
                 ),
-                positional_args   = positional_args,
-                list_star_arg     = None,
-                dict_star_arg     = None,
-                pairs             = pairs,
-                source_ref        = source_ref
+                positional_args = positional_args,
+                list_star_arg   = None,
+                dict_star_arg   = None,
+                pairs           = dict_star_arg,
+                source_ref      = source_ref
             )
 
         return CPythonExpressionBuiltinDict(
             pos_arg    = positional_args[0] if positional_args else None,
-            pairs      = pairs,
+            pairs      = dict_star_arg,
             source_ref = node.getSourceReference()
         )
 
@@ -218,9 +226,10 @@ def chr_extractor( node ):
 
 def ord_extractor( node ):
     return BuiltinOptimization.extractBuiltinArgs(
-        node          = node,
-        builtin_class = CPythonExpressionBuiltinOrd,
-        builtin_spec  = BuiltinOptimization.builtin_ord_spec
+        node                = node,
+        builtin_class       = CPythonExpressionBuiltinOrd,
+        builtin_spec        = BuiltinOptimization.builtin_ord_spec,
+        empty_special_class = CPythonExpressionBuiltinOrd0
     )
 
 def bin_extractor( node ):
@@ -259,10 +268,19 @@ def repr_extractor( node ):
     )
 
 def range_extractor( node ):
+    def selectRangeBuiltin( low, high, step, source_ref ):
+        if high is None:
+            return CPythonExpressionBuiltinRange1( low, source_ref )
+        elif step is None:
+            return CPythonExpressionBuiltinRange2( low, high, source_ref )
+        else:
+            return CPythonExpressionBuiltinRange3( low, high, step, source_ref )
+
     return BuiltinOptimization.extractBuiltinArgs(
-        node          = node,
-        builtin_class = CPythonExpressionBuiltinRange,
-        builtin_spec  = BuiltinOptimization.builtin_range_spec
+        node                = node,
+        builtin_class       = selectRangeBuiltin,
+        empty_special_class = CPythonExpressionBuiltinRange0,
+        builtin_spec        = BuiltinOptimization.builtin_range_spec
     )
 
 def len_extractor( node ):
@@ -300,6 +318,17 @@ def str_extractor( node ):
         builtin_spec  = BuiltinOptimization.builtin_str_spec
     )
 
+if python_version < 300:
+    from nuitka.nodes.BuiltinTypeNodes import CPythonExpressionBuiltinUnicode
+
+    def unicode_extractor( node ):
+        return BuiltinOptimization.extractBuiltinArgs(
+            node          = node,
+            builtin_class = CPythonExpressionBuiltinUnicode,
+            builtin_spec  = BuiltinOptimization.builtin_unicode_spec
+        )
+
+
 def bool_extractor( node ):
     return BuiltinOptimization.extractBuiltinArgs(
         node          = node,
@@ -314,7 +343,7 @@ def int_extractor( node ):
         builtin_spec  = BuiltinOptimization.builtin_int_spec
     )
 
-if getPythonVersion() < 300:
+if python_version < 300:
     from nuitka.nodes.BuiltinTypeNodes import CPythonExpressionBuiltinLong
 
     def long_extractor( node ):
@@ -325,32 +354,35 @@ if getPythonVersion() < 300:
         )
 
 def globals_extractor( node ):
-    assert node.isEmptyCall()
-
-    return CPythonExpressionBuiltinGlobals(
-        source_ref = node.getSourceReference()
+    return BuiltinOptimization.extractBuiltinArgs(
+        node          = node,
+        builtin_class = CPythonExpressionBuiltinGlobals,
+        builtin_spec  = BuiltinOptimization.builtin_globals_spec
     )
 
 def locals_extractor( node ):
-    assert node.isEmptyCall()
-
     # Note: Locals on the module level is really globals.
     provider = node.getParentVariableProvider()
 
     if provider.isModule():
-        return CPythonExpressionBuiltinGlobals(
-            source_ref = node.getSourceReference()
+        return BuiltinOptimization.extractBuiltinArgs(
+            node          = node,
+            builtin_class = CPythonExpressionBuiltinGlobals,
+            builtin_spec  = BuiltinOptimization.builtin_locals_spec
         )
     else:
-        return CPythonExpressionBuiltinLocals(
-            source_ref = node.getSourceReference()
+        return BuiltinOptimization.extractBuiltinArgs(
+            node          = node,
+            builtin_class = CPythonExpressionBuiltinLocals,
+            builtin_spec  = BuiltinOptimization.builtin_locals_spec
         )
 
-
 def execfile_extractor( node ):
     def wrapExpressionBuiltinExecfileCreation( filename, globals_arg, locals_arg, source_ref ):
 
-        if node.getParentVariableProvider().isExpressionClassBody():
+        provider = node.getParentVariableProvider()
+
+        if provider.isExpressionFunctionBody() and provider.isClassDictCreation():
             # In a case, the copy-back must be done and will only be done correctly by
             # the code for exec statements.
 
@@ -358,9 +390,12 @@ def execfile_extractor( node ):
         else:
             use_call = CPythonExpressionBuiltinExecfile
 
+        if provider.isExpressionFunctionBody():
+            provider.markAsExecContaining()
+
         return use_call(
-            source_code = CPythonExpressionFunctionCall(
-                called_expression = CPythonExpressionAttributeLookup(
+            source_code = CPythonExpressionCall(
+                called          = CPythonExpressionAttributeLookup(
                     expression     = CPythonExpressionBuiltinOpen(
                         filename   = filename,
                         mode       = CPythonExpressionConstantRef(
@@ -373,11 +408,11 @@ def execfile_extractor( node ):
                     attribute_name = "read",
                     source_ref     = source_ref
                 ),
-                positional_args  = (),
-                pairs            = (),
-                list_star_arg    = None,
-                dict_star_arg    = None,
-                source_ref       = source_ref
+                positional_args = (),
+                pairs           = (),
+                list_star_arg   = None,
+                dict_star_arg   = None,
+                source_ref      = source_ref
             ),
             globals_arg = globals_arg,
             locals_arg  = locals_arg,
@@ -421,6 +456,69 @@ def open_extractor( node ):
         builtin_spec  = BuiltinOptimization.builtin_open_spec
     )
 
+def super_extractor( node ):
+
+
+    # Need to accept type and object as keyword argument, that is just the API of super,
+    # pylint: disable=W0622
+    def wrapSuperBuiltin( type, object, source_ref ):
+        if type is None and python_version >= 300:
+            provider = node.getParentVariableProvider()
+
+            type = CPythonExpressionVariableRef(
+                variable_name = "__class__",
+                source_ref    = source_ref
+            )
+
+            # Ought to be already closure taken.
+            type.setVariable(
+                provider.getVariableForReference(
+                    variable_name = "__class__"
+                )
+            )
+
+            if not type.getVariable().isClosureReference():
+                return makeRaiseExceptionReplacementExpression(
+                    expression      = node,
+                    exception_type  = "SystemError",
+                    exception_value = "super(): __class__ cell not found",
+                )
+
+            if object is None and provider.getParameters().getArgumentCount() > 0:
+                par1_name = provider.getParameters().getArgumentNames()[0]
+                # TODO: Nested first argument would kill us here, need a test for that.
+
+                object = CPythonExpressionVariableRef(
+                    variable_name = par1_name,
+                    source_ref    = source_ref
+                )
+
+                object.setVariable(
+                    node.getParentVariableProvider().getVariableForReference(
+                        variable_name = par1_name
+                    )
+                )
+
+                if not object.getVariable().isParameterVariable():
+                    return makeRaiseExceptionReplacementExpression(
+                        expression      = node,
+                        exception_type  = "SystemError",
+                        exception_value = "super(): __class__ cell not found",
+                    )
+
+
+        return CPythonExpressionBuiltinSuper(
+            super_type   = type,
+            super_object = object,
+            source_ref   = source_ref
+        )
+
+    return BuiltinOptimization.extractBuiltinArgs(
+        node          = node,
+        builtin_class = wrapSuperBuiltin,
+        builtin_spec  = BuiltinOptimization.builtin_super_spec
+    )
+
 _dispatch_dict = {
     "globals"    : globals_extractor,
     "locals"     : locals_extractor,
@@ -448,11 +546,13 @@ _dispatch_dict = {
     "int"        : int_extractor,
     "repr"       : repr_extractor,
     "len"        : len_extractor,
-    "open"       : open_extractor
+    "open"       : open_extractor,
+    "super"      : super_extractor
 }
 
-if getPythonVersion() < 300:
+if python_version < 300:
     _dispatch_dict[ "long" ] = long_extractor
+    _dispatch_dict[ "unicode" ] = unicode_extractor
 
 
 def computeBuiltinCall( call_node, called ):
@@ -464,9 +564,6 @@ def computeBuiltinCall( call_node, called ):
         if new_node is None:
             return call_node, None, None
 
-            # TODO: Actually returning None should not be allowed at this point.
-            raise AssertionError( "None is not allowed to return", _dispatch_dict[ builtin_name ] )
-
         tags = set( [ "new_builtin" ] )
 
         if new_node.isExpressionBuiltinImport():
@@ -475,7 +572,7 @@ def computeBuiltinCall( call_node, called ):
         elif new_node.isExpressionBuiltin() or new_node.isStatementExec():
             tags = "new_builtin"
             message = "Replaced call to builtin %s with builtin call." % new_node.kind
-        elif new_node.isExpressionFunctionCall():
+        elif new_node.isExpressionCall() or new_node.isExpressionRaiseException():
             tags = "new_raise"
             message = "Replaced call to builtin %s with exception raising call." % new_node.kind
         elif new_node.isExpressionOperationUnary():
