@@ -21,7 +21,7 @@
 
 print "Output with newline."
 print "Output", "with", "newline."
-print "Output ", "with ", "newline."
+print "Output trailing spaces ", "with ", "newline."
 print "Output ",
 print "with ",
 print "newline."
@@ -39,6 +39,6 @@ a = "partial print"
 try:
     print a, b
 except Exception, e:
-    print "then occured", e
+    print "then occured", repr(e)
 
 print "No newline at the end",
