@@ -399,12 +399,52 @@ def simpleFunction50():
    for x in range( 2 ):
       r = list( f() )
 
+def simpleFunction51():
+   g = ( x for x in range(9) )
+
+   try:
+      g.throw( ValueError, 9 )
+   except ValueError, e:
+      pass
+
+def simpleFunction52():
+   g = ( x for x in range(9) )
+
+   try:
+      g.throw( ValueError( 9 ) )
+   except ValueError, e:
+      pass
+
+def simpleFunction53():
+   g = ( x for x in range(9) )
+
+   try:
+      g.send( 9 )
+   except TypeError, e:
+      pass
+
+def simpleFunction54():
+   g = ( x for x in range(9) )
+   g.next()
+
+   try:
+      g.send( 9 )
+   except TypeError, e:
+      pass
+
+
+def simpleFunction55():
+   g = ( x for x in range(9) )
+
+   try:
+      g.close()
+   except ValueError, e:
+      pass
 
 x = 17
 
-def checkReferenceCount( checked_function, max_rounds = 4 ):
-   if sys.version_info[0] < 3:
-      sys.exc_clear()
+def checkReferenceCount( checked_function, max_rounds = 10 ):
+   assert sys.exc_info() == ( None, None, None ), sys.exc_info()
 
    print checked_function.func_name + ":",
 
@@ -417,8 +457,7 @@ def checkReferenceCount( checked_function, max_rounds = 4 ):
 
       checked_function()
 
-      if sys.version_info[0] < 3:
-         sys.exc_clear()
+      assert sys.exc_info() == ( None, None, None ), sys.exc_info()
 
       gc.collect()
 
@@ -430,8 +469,7 @@ def checkReferenceCount( checked_function, max_rounds = 4 ):
    else:
       print "FAILED", ref_count1, ref_count2
 
-   if sys.version_info[0] < 3:
-      sys.exc_clear()
+   assert sys.exc_info() == ( None, None, None ), sys.exc_info()
 
    gc.collect()
 
@@ -486,3 +524,8 @@ checkReferenceCount( simpleFunction47 )
 checkReferenceCount( simpleFunction48 )
 checkReferenceCount( simpleFunction49 )
 checkReferenceCount( simpleFunction50 )
+checkReferenceCount( simpleFunction51 )
+checkReferenceCount( simpleFunction52 )
+checkReferenceCount( simpleFunction53 )
+checkReferenceCount( simpleFunction54 )
+checkReferenceCount( simpleFunction55 )
