@@ -19,4 +19,4 @@
 import some_package.DeepChild
 import some_package.deep_package.DeepDeepChild
 
-print "Done."
+print( "Done." )
