@@ -16,10 +16,10 @@
 #     limitations under the License.
 #
 
-print "*** Child1: Begin"
+print( "*** Child1: Begin" )
 
 from . import Child3 as localname
 
-print "*** Child1: Imported Child3", localname
+print( "*** Child1: Imported Child3", localname )
 
-print "*** Child1: End"
+print( "*** Child1: End" )
