@@ -72,7 +72,7 @@ for filename in sorted( os.listdir( "." ) ):
         active = True
 
     if active:
-        if filename not in ( "module_exits", "package_missing_init" ):
+        if filename not in ( "module_exits", "package_missing_init", "main_raises", "main_raises2" ):
             extra_flags = [ "expect_success" ]
         else:
             extra_flags = [ "expect_failure" ]
@@ -95,8 +95,11 @@ for filename in sorted( os.listdir( "." ) ):
         for filename_main in os.listdir( filename ):
             if filename_main.endswith( "Main.py" ):
                 break
+
+            if filename_main.endswith( "Main" ):
+                break
         else:
-            assert False, filename
+            sys.exit( "Error, no file ends with 'Main.py' or 'Main' in %s, incomplete test case" % filename )
 
         result = subprocess.call(
             "%s %s %s silent %s" % (
