@@ -21,8 +21,8 @@ def starImportFailure():
 
     try:
         sys
-        print "but it does not"
+        print( "but it does not" )
     except NameError:
-        print "and it does"
+        print( "and it does" )
 
-print "Star import needs to respect __all__", starImportFailure()
+print( "Star import needs to respect __all__", starImportFailure() )
